# Copyright (c) Microsoft Corporation.  All rights reserved.
# Licensed under the MIT License.  See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import os
import unittest

import numpy as np
import onnx
from onnx import TensorProto, helper
from parity_utilities import find_transformers_source

if find_transformers_source():
    from fusion_options import FusionOptions
    from onnx_model import OnnxModel
    from optimizer import optimize_model
else:
    from onnxruntime.transformers.fusion_options import FusionOptions
    from onnxruntime.transformers.onnx_model import OnnxModel
    from onnxruntime.transformers.optimizer import optimize_model


def float_tensor(name: str, shape: list[int], random=False):
    low = 0.0
    high = 1.0
    total_elements = 1
    for x in shape:
        total_elements *= x
    weights = [np.random.uniform(low, high) for _ in range(total_elements)] if random else [1.0] * total_elements
    return helper.make_tensor(name, TensorProto.FLOAT, shape, weights)


class TestSimplifiedLayerNormFusion(unittest.TestCase):
    def setUp(self):
        self.vocab_size = 5
        self.batch_size = 2
        self.sequence_length = 8
        self.hidden_size = 16
        self.epsilon = 0.000009999999747378752

    def verify_fusion(self, expected_model_path, original_model_path):
        expected_model = OnnxModel(onnx.load(expected_model_path))
        expected_model.topological_sort(is_deterministic=True)

        options = FusionOptions("gpt2")
        optimized_model = optimize_model(original_model_path, optimization_options=options)
        optimized_model.topological_sort(is_deterministic=True)

        self.assertTrue(str(expected_model.model.graph), str(optimized_model.model.graph))

    def create_initializers(self, use_embed_weight: bool = False):
        initializers = [
            helper.make_tensor("Two", TensorProto.FLOAT, [1], np.array([2], dtype=np.float32)),
            helper.make_tensor("epsilon", TensorProto.FLOAT, [1], np.array([self.epsilon], dtype=np.float32)),
            helper.make_tensor("One", TensorProto.FLOAT, [1], np.array([1], dtype=np.float32)),
            float_tensor("scale", [self.hidden_size]),
        ]
        if use_embed_weight:
            initializers = [  # noqa: RUF005
                float_tensor("embed_weight", [self.vocab_size, self.hidden_size])
            ] + initializers
        return initializers

    def create_inputs_and_outputs(self, start_node_type: str):
        inputs, start_node = None, None
        if start_node_type == "Add":
            start_node = helper.make_node(
                "Add",
                inputs=["input_0", "input_1"],
                outputs=["D"],
                name="Add_0",
            )
            input_0 = helper.make_tensor_value_info(
                "input_0",
                TensorProto.FLOAT,
                [self.batch_size, self.sequence_length, self.hidden_size],
            )
            input_1 = helper.make_tensor_value_info(
                "input_1",
                TensorProto.FLOAT,
                [self.batch_size, self.sequence_length, self.hidden_size],
            )
            inputs = [input_0, input_1]
        elif start_node_type == "Gather":
            start_node = helper.make_node(
                "Gather",
                inputs=["embed_weight", "input_0"],
                outputs=["D"],
                name="Gather_0",
            )
            input_0 = helper.make_tensor_value_info(
                "input_0",
                TensorProto.INT64,
                [self.batch_size, self.sequence_length],
            )
            inputs = [input_0]
        else:
            # start_node_type is a graph input
            assert start_node_type == "GraphInput"
            input_0 = helper.make_tensor_value_info(
                "D",
                TensorProto.FLOAT,
                [self.batch_size, self.sequence_length, self.hidden_size],
            )
            inputs = [input_0]

        outputs = [
            helper.make_tensor_value_info(
                "output_0",
                TensorProto.FLOAT,
                [self.batch_size, self.sequence_length, self.hidden_size],
            )
        ]
        return inputs, outputs, start_node

    def create_fused_model(self, start_node_type: str, initializers: list[TensorProto]):
        inputs, outputs, start_node = self.create_inputs_and_outputs(start_node_type)

        sln_node = helper.make_node(
            "SimplifiedLayerNormalization",
            inputs=[start_node.output[0] if start_node is not None else "D", initializers[0].name],
            outputs=[outputs[0].name],
            axis=-1,
            epsilon=initializers[2].float_data[0],
            stash_type=1,
        )

        graph = helper.make_graph(
            nodes=[sln_node] + ([] if start_node is None else [start_node]),
            name="SimplifiedLayerNorm_Graph",
            inputs=inputs,
            outputs=outputs,
            initializer=initializers,
        )
        opset_import = helper.make_opsetid(domain="com.microsoft", version=1)
        model = helper.make_model(graph, opset_imports=[opset_import])
        return model

    # Notation follows https://onnx.ai/onnx/operators/onnx__LayerNormalization.html#summary
    def create_test_model(self, start_node_type: str, first_parent_idx: int, initializers: list[TensorProto]):
        end_node = helper.make_node(
            "Mul",
            inputs=["scale", "Normalized"] if first_parent_idx == 1 else ["Normalized", "scale"],
            outputs=["output_0"],
            name="Mul_1",
        )
        mul_node = helper.make_node(
            "Mul",
            inputs=["D", "InvStdDev"],
            outputs=["Normalized"],
            name="Mul_0",
        )
        div_node = helper.make_node(
            "Div",
            inputs=["One", "StdDev"],
            outputs=["InvStdDev"],
            name="Div_0",
        )
        sqrt_node = helper.make_node(
            "Sqrt",
            inputs=["VarEps"],
            outputs=["StdDev"],
            name="Sqrt_0",
        )
        add_node = helper.make_node(
            "Add",
            inputs=["Var", "epsilon"],
            outputs=["VarEps"],
            name="Add_1",
        )
        reducemean_node = helper.make_node(
            "ReduceMean",
            inputs=["DD"],
            outputs=["Var"],
            name="ReduceMean_0",
        )
        pow_node = helper.make_node(
            "Pow",
            inputs=["D", "Two"],
            outputs=["DD"],
            name="Pow_0",
        )

        inputs, outputs, start_node = self.create_inputs_and_outputs(start_node_type)

        main_nodes = [pow_node, reducemean_node, add_node, sqrt_node, div_node, mul_node, end_node]
        graph = helper.make_graph(
            nodes=main_nodes + ([] if start_node is None else [start_node]),
            name="SimplifiedLayerNorm_Graph",
            inputs=inputs,
            outputs=outputs,
            initializer=initializers,
        )
        opset_import = helper.make_opsetid(domain="com.microsoft", version=1)
        model = helper.make_model(graph, opset_imports=[opset_import])
        return model

    def check_models(self, start_node_type: str, first_parent_idx: int, initializers: list[TensorProto]):
        expected_model_filename = "expected_model.onnx"
        expected_model = self.create_fused_model(start_node_type, initializers)
        onnx.save(expected_model, expected_model_filename)

        original_model_filename = "original_model.onnx"
        original_model = self.create_test_model(start_node_type, first_parent_idx, initializers)
        onnx.save(original_model, original_model_filename)

        self.verify_fusion(expected_model_filename, original_model_filename)
        os.remove(expected_model_filename)
        os.remove(original_model_filename)

    # sim_ln_nodes_1
    def test_simplified_layernorm_add_idx1(self):
        start_node_type = "Add"
        first_parent_idx = 1
        initializers = self.create_initializers()
        self.check_models(start_node_type, first_parent_idx, initializers)

    # sim_ln_nodes_2
    def test_simplified_layernorm_gather_idx1(self):
        start_node_type = "Gather"
        first_parent_idx = 1
        initializers = self.create_initializers(use_embed_weight=True)
        self.check_models(start_node_type, first_parent_idx, initializers)

    # sim_ln_nodes_3
    def test_simplified_layernorm_add_idx0(self):
        start_node_type = "Add"
        first_parent_idx = 0
        initializers = self.create_initializers()
        self.check_models(start_node_type, first_parent_idx, initializers)

    # sim_ln_nodes_4
    def test_simplified_layernorm_gather_graph_input(self):
        start_node_type = "GraphInput"
        first_parent_idx = 0
        initializers = self.create_initializers()
        self.check_models(start_node_type, first_parent_idx, initializers)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

import argparse
import json
import os
import subprocess
import sys
from collections import namedtuple

SCRIPT_DIR = os.path.realpath(os.path.dirname(__file__))


def parse_args():
    parser = argparse.ArgumentParser(description="Runs BERT performance tests.")
    parser.add_argument("--binary_dir", required=True, help="Path to the ORT binary directory.")
    parser.add_argument("--training_data_root", required=True, help="Path to the training data root directory.")
    parser.add_argument("--model_root", required=True, help="Path to the model root directory.")
    parser.add_argument(
        "--gpu_sku",
        choices=["V100_16G", "MI100_32G"],
        default="V100_16G",
        required=False,
        help="GPU model (e.g. V100_16G, MI100_32G).",
    )
    return parser.parse_args()


# using the same params from "GitHub Master Merge Schedule" in OneNotes
def main():
    args = parse_args()

    Config = namedtuple(
        "Config", ["use_mixed_precision", "max_seq_length", "batch_size", "max_predictions_per_seq", "expected_perf"]
    )
    configs = {}
    configs["V100_16G"] = [
        Config(True, 128, 76, 20, -1.0),
        Config(True, 512, 11, 80, -1.0),
        Config(False, 128, 39, 20, -1.0),
        Config(False, 512, 6, 80, -1.0),
    ]

    configs["MI100_32G"] = [
        Config(True, 128, 128, 20, 240),
    ]

    # run BERT training
    for c in configs[args.gpu_sku]:
        model = "bert-large-uncased_L_24_H_1024_A_16_V_30528_S_512_Dp_0.1_optimized_layer_norm_opset12"
        precision_prefix = "fp16" if c.use_mixed_precision else "fp32"
        print(
            "######## testing name - "
            + ("fp16-" if c.use_mixed_precision else "fp32-")
            + str(c.max_seq_length)
            + " ##############"
        )
        cmds = [
            os.path.join(args.binary_dir, "onnxruntime_training_bert"),
            "--model_name",
            os.path.join(args.model_root, f"nv/bert-large/{model}"),
            "--train_data_dir",
            os.path.join(args.training_data_root, str(c.max_seq_length), "books_wiki_en_corpus/train"),
            "--test_data_dir",
            os.path.join(args.training_data_root, str(c.max_seq_length), "books_wiki_en_corpus/test"),
            "--train_batch_size",
            str(c.batch_size),
            "--mode",
            "train",
            "--max_seq_length",
            str(c.max_seq_length),
            "--num_train_steps",
            "640",
            "--display_loss_steps",
            "5",
            "--optimizer",
            "Lamb",
            "--learning_rate",
            "3e-3",
            "--warmup_ratio",
            "0.2843",
            "--warmup_mode",
            "Poly",
            "--gradient_accumulation_steps",
            "1",
            "--max_predictions_per_seq",
            str(c.max_predictions_per_seq),
            "--lambda",
            "0",
            "--use_nccl",
            "--perf_output_dir",
            os.path.join(SCRIPT_DIR, "results"),
        ]

        if c.use_mixed_precision:
            (cmds.append("--use_mixed_precision"),)
            (cmds.append("--allreduce_in_fp16"),)

        subprocess.run(cmds).check_returncode()  # noqa: PLW1510
        if c.expected_perf > 0.0:
            json_filename = (
                f"onnxruntime_perf_metrics_{model}.onnx_bert_{precision_prefix}_{c.max_seq_length}_Lamb.json"
            )
            with open(os.path.join(SCRIPT_DIR, "results", json_filename)) as json_file:
                results = json.load(json_file)
                assert results["EndToEndThroughput"] > 0.98 * c.expected_perf

    return 0


if __name__ == "__main__":
    sys.exit(main())



import onnx
from onnx import TensorProto, helper


# Since NNAPI EP handles Reshape and Flatten differently,
# Please see ReshapeOpBuilder::CanSkipReshape in <repo_root>/onnxruntime/core/providers/nnapi/nnapi_builtin/builders/op_builder.cc
# We have a separated test for these skip reshape scenarios
def GenerateModel(model_name):  # noqa: N802
    nodes = [
        helper.make_node("Flatten", ["X"], ["Flatten_1_Y"], "flatten_1"),
        helper.make_node("MatMul", ["Flatten_1_Y", "MatMul_B"], ["MatMul_Y"], "matmul"),
        helper.make_node("Reshape", ["Y", "Reshape_1_shape"], ["Reshape_1_Y"], "reshape_1"),
        helper.make_node("Gemm", ["Reshape_1_Y", "Gemm_B"], ["Gemm_Y"], "gemm"),
        helper.make_node("Reshape", ["MatMul_Y", "Reshape_2_shape"], ["Reshape_2_Y"], "reshape_2"),
        helper.make_node("Flatten", ["Gemm_Y"], ["Flatten_2_Y"], "flatten_2", axis=0),
        helper.make_node("Add", ["Reshape_2_Y", "Flatten_2_Y"], ["Z"], "add"),
    ]

    initializers = [
        helper.make_tensor("Reshape_1_shape", TensorProto.INT64, [2], [3, 4]),
        helper.make_tensor("Reshape_2_shape", TensorProto.INT64, [2], [1, 6]),
        helper.make_tensor(
            "Gemm_B",
            TensorProto.FLOAT,
            [4, 2],
            [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0],
        ),
        helper.make_tensor("MatMul_B", TensorProto.FLOAT, [2, 3], [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]),
    ]

    inputs = [
        helper.make_tensor_value_info("X", TensorProto.FLOAT, [2, 1, 2]),
        helper.make_tensor_value_info("Y", TensorProto.FLOAT, [3, 2, 2]),
    ]

    graph = helper.make_graph(
        nodes,
        "NNAPI_Reshape_Flatten_Test",
        inputs,
        [helper.make_tensor_value_info("Z", TensorProto.FLOAT, [1, 6])],
        initializers,
    )

    model = helper.make_model(graph)
    onnx.save(model, model_name)


if __name__ == "__main__":
    GenerateModel("nnapi_reshape_flatten_test.onnx")



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

"""This file is used to generate test data for Adam optimizer tests in
orttraining/orttraining/test/training_ops/cuda/optimizer/adamw_test.cc."""

import torch


class SingleParameterModule(torch.nn.Module):
    """A dummy module containing only one trainable parameter."""

    def __init__(self, input_size, hidden_size):
        super().__init__()
        self.fc1 = torch.nn.Linear(input_size, hidden_size, bias=False)

    def forward(self, input1):
        """Module forward call."""
        out = self.fc1(input1)
        return out


class MultipleParametersModule(torch.nn.Module):
    """A dummy module containing multiple trainable parameters."""

    def __init__(self, input_size, hidden_size, num_classes):
        super().__init__()
        self.fc1 = torch.nn.Linear(input_size, hidden_size)
        self.relu = torch.nn.ReLU()
        self.fc2 = torch.nn.Linear(hidden_size, num_classes)

    def forward(self, input1):
        """Module forward call."""
        out = self.fc1(input1)
        out = self.relu(out)
        out = self.fc2(out)
        return out


def generate_adamw_test_data(seed, _model_setup_func, data_func, train_step_count, adam_mode, json_file_name, device):
    """Generate test data using specified model/data and other configs."""

    is_cuda = device == "cuda"

    def _sync_stream():
        if is_cuda is True:
            torch.cuda.synchronize()

    def _run_step(model, input, target):
        prediction = model(input)
        criterion = torch.nn.MSELoss()
        loss = criterion(prediction, target)
        loss.backward()

    def _torch_tensor_to_str(torch_tensor):
        """Torch tensor to string."""
        return torch_tensor.detach().cpu().numpy().tolist()

    def _build_param_index_to_name_mapping(model, map_result):
        """Build index to name mapping, which is used to retrieve data from optimizer group."""
        for index, param in enumerate(model.named_parameters()):
            map_result[index] = param[0]

    torch.manual_seed(seed)

    # Prepare model, zero gradient.
    pt_model = _model_setup_func()
    pt_model.zero_grad()

    # Prepare optimizer.
    adamw_optimizer = None
    if adam_mode == 0:
        adamw_optimizer = torch.optim.AdamW(pt_model.parameters(), lr=1e-3)
    elif adam_mode == 1:
        from transformers import AdamW

        adamw_optimizer = AdamW(pt_model.parameters(), lr=1e-3)
    else:
        raise ValueError(f"invalid adam_model: {adam_mode}")

    param_index_to_name_mapping = {}
    _build_param_index_to_name_mapping(pt_model, param_index_to_name_mapping)

    # Dump the optimizer configs, our adam tests should align with these.
    for group in adamw_optimizer.param_groups:
        print(
            f"beta1, beta2 : {group['betas']}, lr={group['lr']},"
            " weight_decay={group['weight_decay']}, eps={group['eps']})"
        )

    p_dict = {}
    g_dict = {}
    m1_dict = {}
    m2_dict = {}
    for step in range(train_step_count):
        input, target = data_func()
        _run_step(pt_model, input, target)
        _sync_stream()

        for name, param in pt_model.named_parameters():
            if name not in p_dict:
                p_dict[name] = []
                g_dict[name] = []
                m1_dict[name] = []
                m2_dict[name] = []

            # Collect flattened parameter data.
            p_dict[name].append(_torch_tensor_to_str(param.view(-1)))
            if step == 0:
                print(f"Weight name: {name}, shape: {param.shape}, data type: {param.dtype}")

            if step != train_step_count - 1:
                # Collect flattened gradient data.
                # Skip collecting the last step's gradients.
                g_dict[name].append(_torch_tensor_to_str(param.grad.view(-1)))

        _sync_stream()

        for group in adamw_optimizer.param_groups:
            for p_index, param in enumerate(group["params"]):
                state = adamw_optimizer.state[param]
                name = param_index_to_name_mapping[p_index]
                # Collect flattened optimizer state data.
                if len(state) == 0:
                    m1_dict[name].append(_torch_tensor_to_str(torch.zeros_like(param).view(-1)))
                    m2_dict[name].append(_torch_tensor_to_str(torch.zeros_like(param).view(-1)))
                else:
                    m1_dict[name].append(_torch_tensor_to_str(state["exp_avg"].view(-1)))
                    m2_dict[name].append(_torch_tensor_to_str(state["exp_avg_sq"].view(-1)))

        adamw_optimizer.step()
        adamw_optimizer.zero_grad()

    _sync_stream()
    data = {
        "Parameters": p_dict,
        "Gradients": g_dict,
        "Momentum1s": m1_dict,
        "Momentum2s": m2_dict,
    }
    import json
    import os

    directory = device
    if not os.path.exists(directory):
        os.makedirs(directory)

    with open(os.path.join(directory, json_file_name), "w", encoding="utf-8") as f:
        json.dump(data, f, ensure_ascii=False, indent=4)


def generate_adamw_single_weight_tests(adam_mode, run_step_count, device):
    """Generate test data using specified mode of adamw."""
    seed = 8888
    batch_size, dimension_in, dimension_hidden = 2, 2, 3

    def _model_setup_func():
        pt_model = SingleParameterModule(dimension_in, dimension_hidden).to(device)
        return pt_model

    def _data_func():
        input = torch.randn(batch_size, dimension_in, device=device, dtype=torch.float32)
        target = torch.randn(batch_size, dimension_hidden, device=device, dtype=torch.float32)
        return input, target

    json_file_name = f"adamw_test_single_weight_mode_{adam_mode}.json"
    generate_adamw_test_data(seed, _model_setup_func, _data_func, run_step_count, adam_mode, json_file_name, device)


def generate_adamw_multiple_weights_tests(adam_mode, run_step_count, device):
    """Generate test data using specified mode of adamw."""
    seed = 6666
    batch_size, dimension_in, dimension_hidden, dim_out = 2, 2, 3, 2

    def _model_setup_func():
        pt_model = MultipleParametersModule(dimension_in, dimension_hidden, dim_out).to(device)
        return pt_model

    def data_func():
        input = torch.randn(batch_size, dimension_in, device=device, dtype=torch.float32)
        target = torch.randn(batch_size, dim_out, device=device, dtype=torch.float32)
        return input, target

    json_file_name = f"adamw_test_multiple_weights_mode_{adam_mode}.json"
    generate_adamw_test_data(seed, _model_setup_func, data_func, run_step_count, adam_mode, json_file_name, device)


def main():
    """Main entry."""
    device_candidates = ["cuda", "cpu"]
    test_data_step_count = 11
    for device in device_candidates:
        for adam_mode in range(2):
            generate_adamw_single_weight_tests(adam_mode, test_data_step_count, device)
            generate_adamw_multiple_weights_tests(adam_mode, test_data_step_count, device)


if __name__ == "__main__":
    main()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
# --------------------------------------------------------------------------

import argparse
from dataclasses import dataclass

import numpy as np
from benchmark import BenchmarkOp, add_arguments


@dataclass
class OpParam:
    dim1: int
    dim2: int
    dim3: int
    k: int

    data_type: type


class BenchmarkTopK(BenchmarkOp):
    def __init__(self, args):
        BenchmarkOp.__init__(self, args)

    @classmethod
    def create_inputs_outputs(cls, op_param):
        np.random.seed(0)
        input_data = np.random.rand(op_param.dim1, op_param.dim2, op_param.dim3).astype(op_param.data_type)
        dim_k = np.array([op_param.k]).astype(np.int64)
        values = np.random.rand(op_param.dim1, op_param.dim2, op_param.k).astype(op_param.data_type)
        indices = np.random.rand(op_param.dim1, op_param.dim2, op_param.k).astype(np.int64)
        inputs = {"input": input_data, "k": dim_k}
        outputs = {"values": values, "indices": indices}
        return inputs, outputs

    def create_cases(self):
        # attributes of model: axis=-1, largest=1, sorted=1
        model = "models/topk_fp16.onnx" if self.args.precision == "fp16" else "models/topk_fp32.onnx"
        data_type = np.float16 if self.args.precision == "fp16" else np.float32

        # change here to test your data shape
        self.add_case(OpParam(1, 32, 32, 2, data_type), model)

    @classmethod
    def case_profile(cls, op_param, time):
        profile = f"(dim1 dim2 dim3) = ({op_param.dim1} {op_param.dim2} {op_param.dim3}), k = {op_param.k}, {time * 1000:7.4f} us"
        return profile


def main():
    parser = argparse.ArgumentParser()
    add_arguments(parser)
    args = parser.parse_args()
    bm = BenchmarkTopK(args)
    bm.benchmark()


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
# _torch_module_pytorch.py

from collections import OrderedDict
from collections.abc import Callable, Iterator
from typing import Optional, TypeVar

import torch

from ._torch_module_interface import TorchModuleInterface

T = TypeVar("T", bound="torch.nn.Module")


class TorchModulePytorch(TorchModuleInterface):
    def __init__(self, module: torch.nn.Module):
        super().__init__(module)
        self._original_module = module

    def _apply(self, fn):
        self._original_module._apply(fn)
        return self

    def apply(self: T, fn: Callable[[T], None]) -> T:
        self._original_module.apply(fn)
        return self

    def is_training(self):
        return self._original_module.training and torch.is_grad_enabled()

    def train(self: T, mode: bool = True) -> T:
        self._original_module.train(mode)
        return self

    def state_dict(self, destination=None, prefix="", keep_vars=False):
        return self._original_module.state_dict(destination=destination, prefix=prefix, keep_vars=keep_vars)

    def load_state_dict(self, state_dict: "OrderedDict[str, torch.Tensor]", strict: bool = True):
        return self._original_module.load_state_dict(state_dict, strict=strict)

    def register_buffer(self, name: str, tensor: torch.Tensor | None, persistent: bool = True) -> None:
        self._original_module.register_buffer(name, tensor, persistent=persistent)

    def register_parameter(self, name: str, param: torch.nn.Parameter | None) -> None:
        self._original_module.register_parameter(name, param)

    def get_parameter(self, target: str) -> torch.nn.Parameter:
        return self._original_module.get_parameter(target)

    def get_buffer(self, target: str) -> torch.Tensor:
        return self._original_module.get_buffer(target)

    def parameters(self, recurse: bool = True) -> Iterator[torch.nn.Parameter]:
        yield from self._original_module.parameters(recurse=recurse)

    def named_parameters(self, prefix: str = "", recurse: bool = True) -> Iterator[tuple[str, torch.nn.Parameter]]:
        yield from self._original_module.named_parameters(prefix=prefix, recurse=recurse)

    def buffers(self, recurse: bool = True) -> Iterator[torch.Tensor]:
        yield from self._original_module.buffers(recurse=recurse)

    def named_buffers(self, prefix: str = "", recurse: bool = True) -> Iterator[tuple[str, torch.Tensor]]:
        yield from self._original_module.named_buffers(prefix=prefix, recurse=recurse)

    def _load_from_state_dict(
        self, state_dict, prefix, local_metadata, strict, missing_keys, unexpected_keys, error_msgs
    ):
        self._original_module._load_from_state_dict(
            state_dict, prefix, local_metadata, strict, missing_keys, unexpected_keys, error_msgs
        )

    def named_children(self) -> Iterator[tuple[str, T]]:
        yield from self._original_module.named_children()

    def modules(self) -> Iterator[T]:
        yield from self._original_module.modules()

    def named_modules(self, *args, **kwargs):
        # PyTorch >1.8.1 has an extra arg remove_duplicate that is not present in 1.8.1
        # To support both, use args and kwargs (since user can call the method with only positional args or kwargs)
        yield from self._original_module.named_modules(*args, **kwargs)

    def _replicate_for_data_parallel(self):
        return self._original_module._replicate_for_data_parallel()

    def add_module(self, name: str, module: Optional["Module"]) -> None:  # noqa: F821
        self._original_module.add_module(name, module)

    @TorchModuleInterface.module.getter
    def module(self):
        return self._original_module



#!/usr/bin/env python3
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

from __future__ import annotations

import argparse
import functools
import itertools
import json
import os
import subprocess


@functools.total_ordering
class Version:
    """
    A simple Version class.
    We opt to use this instead of `packaging.version.Version` to avoid depending on the external `packaging` package.
    It only supports integer version components.
    """

    def __init__(self, version_string: str):
        self._components = tuple(int(component) for component in version_string.split("."))

    def __eq__(self, other: Version) -> bool:
        component_pairs = itertools.zip_longest(self._components, other._components, fillvalue=0)
        return all(pair[0] == pair[1] for pair in component_pairs)

    def __lt__(self, other: Version) -> bool:
        component_pairs = itertools.zip_longest(self._components, other._components, fillvalue=0)
        for self_component, other_component in component_pairs:
            if self_component != other_component:
                return self_component < other_component
        return False


def get_simulator_device_info(
    requested_runtime_platform: str = "iOS",
    requested_device_type_product_family: str = "iPhone",
    requested_runtime_version_str: str | None = None,
) -> dict[str, str]:
    """
    Retrieves simulator device information from Xcode.
    This simulator device should be appropriate for running tests on this machine.

    :param requested_runtime_platform: The runtime platform to select.
    :param requested_device_type_product_family: The device type product family to select.
    :param requested_runtime_version_str: The runtime version to select. If unspecified, selects the latest one.

    :return: A dictionary containing information about the selected simulator device.
    """
    requested_runtime_version = (
        Version(requested_runtime_version_str) if requested_runtime_version_str is not None else None
    )

    simctl_proc = subprocess.run(
        ["xcrun", "simctl", "list", "--json", "--no-escape-slashes"],
        text=True,
        capture_output=True,
        check=True,
    )

    simctl_json = json.loads(simctl_proc.stdout)

    # device type id -> device type structure
    device_type_map = {device_type["identifier"]: device_type for device_type in simctl_json["devicetypes"]}

    # runtime id -> runtime structure
    runtime_map = {runtime["identifier"]: runtime for runtime in simctl_json["runtimes"]}

    def runtime_filter(runtime) -> bool:
        if not runtime["isAvailable"]:
            return False

        if runtime["platform"] != requested_runtime_platform:
            return False

        if requested_runtime_version is not None and Version(runtime["version"]) != requested_runtime_version:
            return False

        return True

    def runtime_id_filter(runtime_id: str) -> bool:
        runtime = runtime_map.get(runtime_id)
        if runtime is None:
            return False
        return runtime_filter(runtime)

    def device_type_filter(device_type) -> bool:
        if device_type["productFamily"] != requested_device_type_product_family:
            return False

        return True

    def device_filter(device) -> bool:
        if not device["isAvailable"]:
            return False

        if not device_type_filter(device_type_map[device["deviceTypeIdentifier"]]):
            return False

        return True

    # simctl_json["devices"] is a map of runtime id -> list of device structures
    # expand this into a list of (runtime id, device structure) and filter out invalid entries
    runtime_id_and_device_pairs = []
    for runtime_id, device_list in filter(
        lambda runtime_id_and_device_list: runtime_id_filter(runtime_id_and_device_list[0]),
        simctl_json["devices"].items(),
    ):
        runtime_id_and_device_pairs.extend((runtime_id, device) for device in filter(device_filter, device_list))

    if len(runtime_id_and_device_pairs) == 0:
        raise ValueError("Failed to find requested simulator device info.")

    # sort key - tuple of (runtime version, device type min runtime version)
    # the secondary device type min runtime version value is to treat more recent device types as greater
    def runtime_id_and_device_pair_key(runtime_id_and_device_pair):
        runtime_id, device = runtime_id_and_device_pair

        runtime = runtime_map[runtime_id]
        device_type = device_type_map[device["deviceTypeIdentifier"]]

        return (Version(runtime["version"]), device_type["minRuntimeVersion"])

    selected_runtime_id, selected_device = max(runtime_id_and_device_pairs, key=runtime_id_and_device_pair_key)
    selected_runtime = runtime_map[selected_runtime_id]
    selected_device_type = device_type_map[selected_device["deviceTypeIdentifier"]]

    result = {
        "device_name": selected_device["name"],
        "device_udid": selected_device["udid"],
        "device_type_identifier": selected_device_type["identifier"],
        "device_type_name": selected_device_type["name"],
        "device_type_product_family": selected_device_type["productFamily"],
        "runtime_identifier": selected_runtime["identifier"],
        "runtime_platform": selected_runtime["platform"],
        "runtime_version": selected_runtime["version"],
    }

    return result


def main():
    requested_runtime_version_environment_variable_name = "ORT_GET_SIMULATOR_DEVICE_INFO_REQUESTED_RUNTIME_VERSION"

    parser = argparse.ArgumentParser(description="Gets simulator info from Xcode and prints it in JSON format.")
    parser.add_argument(
        "--requested-runtime-version",
        default=os.environ.get(requested_runtime_version_environment_variable_name, None),
        help="The requested runtime version. "
        f"This may also be specified with the {requested_runtime_version_environment_variable_name} "
        "environment variable. The command line option takes precedence. "
        "An unspecified value means the latest available runtime version.",
    )
    args = parser.parse_args()

    info = get_simulator_device_info(requested_runtime_version_str=args.requested_runtime_version)

    print(json.dumps(info, indent=2))


if __name__ == "__main__":
    main()



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
from __future__ import annotations

import contextlib
import logging
import os
import platform
import re
import shutil
import stat
import subprocess
import tempfile
import urllib.parse
import urllib.request

AZCOPY_VERSION = "10.4.3"

# See here for instructions on getting stable download links:
# https://docs.microsoft.com/en-us/azure/storage/common/storage-use-azcopy-v10#obtain-a-static-download-link
_AZCOPY_DOWNLOAD_URLS = {
    "Linux": "https://azcopyvnext.azureedge.net/release20200501/azcopy_linux_amd64_10.4.3.tar.gz",
    "Darwin": "https://azcopyvnext.azureedge.net/release20200501/azcopy_darwin_amd64_10.4.3.zip",
    "Windows": "https://azcopyvnext.azureedge.net/release20200501/azcopy_windows_amd64_10.4.3.zip",
}

_log = logging.getLogger("util.get_azcopy")


def _check_version(azcopy_path):
    proc = subprocess.run([azcopy_path, "--version"], stdout=subprocess.PIPE, text=True)  # noqa: PLW1510
    match = re.search(r"\d+(?:\.\d+)+", proc.stdout)

    if not match:
        raise RuntimeError("Failed to determine azcopy version.")

    return match.group(0) == AZCOPY_VERSION


def _find_azcopy(start_dir):
    for root, _, file_names in os.walk(start_dir):
        for file_name in file_names:
            if file_name == "azcopy" or file_name == "azcopy.exe":
                return os.path.join(root, file_name)
    raise RuntimeError(f"Failed to azcopy in '{start_dir}'.")


@contextlib.contextmanager
def get_azcopy(local_azcopy_path="azcopy"):
    """
    Creates a context manager that returns a path to a particular version of
    azcopy (specified in AZCOPY_VERSION). Downloads a temporary copy if needed.

    :param local_azcopy_path: Path to a local azcopy to try first.

    Example usage:
        with get_azcopy() as azcopy_path:
            subprocess.run([azcopy_path, "--version"])
    """
    with contextlib.ExitStack() as context_stack:
        azcopy_path = shutil.which(local_azcopy_path)

        if azcopy_path is None or not _check_version(azcopy_path):
            temp_dir = context_stack.enter_context(tempfile.TemporaryDirectory())

            download_url = _AZCOPY_DOWNLOAD_URLS[platform.system()]
            download_basename = urllib.parse.urlsplit(download_url).path.rsplit("/", 1)[-1]
            assert len(download_basename) > 0
            downloaded_path = os.path.join(temp_dir, download_basename)

            _log.info(f"Downloading azcopy from '{download_url}'...")
            urllib.request.urlretrieve(download_url, downloaded_path)

            extracted_path = os.path.join(temp_dir, "azcopy")
            shutil.unpack_archive(downloaded_path, extracted_path)

            azcopy_path = _find_azcopy(extracted_path)

            os.chmod(azcopy_path, stat.S_IXUSR)

        yield azcopy_path



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
# --------------------------------------------------------------------------

import itertools
import warnings
from collections import defaultdict
from typing import Any

import sympy
from onnx import NodeProto, helper

from ._common import AutotuneConfigs, TensorInfo
from ._ir import (
    ComputeNode,
    DropoutNode,
    ElementwiseKernelNode,
    IONode,
    KernelNode,
    ModuleNode,
    OffsetCalculator,
    ReduceForLoopEnd,
    ReduceForLoopStart,
    ReduceKernelNode,
    ReduceNode,
    TensorArg,
)
from ._op_config import is_reduction_node
from ._sorted_graph import SortedGraph
from ._utils import get_reduce_info, sort_reduce_axes, to_torch_tensor


class NodeGroup:
    """
    A NodeGroup contains nodes that can be lowered to a single Triton kernel node.

    """

    def __init__(self, node: NodeProto, reduce_axes: list[int], keep_dims: int, node_arg_infos: dict[str, TensorInfo]):
        self._node_arg_infos = node_arg_infos
        self.nodes_groups: list[Any] = [node]
        self.target_shape: list[sympy.Expr] = self._get_target_shape(node)
        rank = len(self.target_shape)
        self.reduce_axes: list[int] = sort_reduce_axes(reduce_axes, rank)
        x_dims = [self.target_shape[dim] for dim in range(rank) if dim not in self.reduce_axes]
        # x_numel is meant to hint how many rows of tensor will be processed by each kernel.
        # x is same as CUDA block in X direction.
        x_numel: sympy.Expr = sympy.prod(x_dims) if len(x_dims) > 0 else sympy.Integer(1)
        r_dims: list[sympy.Expr] = [self.target_shape[dim] for dim in self.reduce_axes]
        # r_numel is meant to hint how many elements in a row of tensor will be processed by each kernel.
        # r is a abbreviation of reduction, so, it's only used for reduction nodes.
        r_numel: sympy.Expr = sympy.prod(r_dims) if len(r_dims) > 0 else sympy.Integer(1)
        self.autotune_configs: AutotuneConfigs = AutotuneConfigs(
            x_numel, r_numel, len(self.reduce_axes) == 0 or self.reduce_axes[-1] == rank - 1
        )
        self.reduced_args: set[str] = set()
        if keep_dims != 1:
            self.reduced_args.add(node.output[0])

    # Check if shape can be broadcasted to target_shape.
    # For example, [1, 3, 1, 1] can be broadcasted to [1, 3, 5, 7].
    # and we support `keepdims = false``, so [1, 3, 5, 7] is compatible with [1, 3, 5].
    def _compatible_shape(self, shape: list[sympy.Expr], split_if_different: bool) -> bool:
        if split_if_different:
            return shape == self.target_shape
        if len(shape) > len(self.target_shape):
            return False
        shape = [sympy.Integer(1)] * (len(self.target_shape) - len(shape)) + shape
        for axis, dim in enumerate(shape):
            if dim != self.target_shape[axis] and (not dim.is_number or dim != sympy.Integer(1)):
                return False
        return True

    # Only we consider reduction or elementwise nodes.
    # target shape does effect how we block the tensor in a triton kernel
    # for reduction, it's possible to set keepdims=False
    # for element-wise, output shape is always the target shape.
    def _get_target_shape(self, node):
        name = node.input[0] if is_reduction_node(node) else node.output[0]
        return self._node_arg_infos[name].shape

    # Check if a node can be added to this group.
    # a group represents a single kernel.
    # Theoretically, we should fuse as more nodes as possible to benefit most from memory access pattern.
    # But we have to consider the following factors:
    #     1. We have to keep the order of nodes, so that we can't fuse nodes that are not adjacent.
    #     2. The target shape of a group is determined by the first node in the group.
    #       we call it dominators, and it determinate the partition strategy of X_numel/R_numel.
    #       A group can't have multiple dominators.
    def compatible(self, node: NodeProto, reduce_axes: list[int], keep_dims: int, split_if_different: bool) -> bool:
        target_shape = self._get_target_shape(node)
        if is_reduction_node(node):
            # If the following nodes are all elementwise nodes on reduce output shape.
            if len(self.reduce_axes) == 0 and self.target_shape == self._node_arg_infos[node.output[0]].shape:
                return True
            if keep_dims != 1:
                return False
            if (
                len(self.reduce_axes) > 0 and self.reduce_axes != sort_reduce_axes(reduce_axes, len(target_shape))
            ) or self.target_shape != target_shape:
                return False
            return True
        return self._compatible_shape(target_shape, split_if_different)

    # 1. Create a new group with the reduction node.
    # 2. Add this node to the current group.
    def add_node(self, node: NodeProto, reduce_axes: list[int], keep_dims: int):
        if is_reduction_node(node):
            group = NodeGroup(node, reduce_axes, keep_dims, self._node_arg_infos)
            self.nodes_groups.append(group)
            if len(self.reduce_axes) == 0:
                self.target_shape = group.target_shape
                self.reduce_axes = group.reduce_axes
                self.autotune_configs = group.autotune_configs
                if keep_dims != 1:
                    for idx in range(len(self.nodes_groups) - 1):
                        self.reduced_args.update(self.nodes_groups[idx].input)
                        self.reduced_args.update(self.nodes_groups[idx].output)
            return group
        self.nodes_groups.append(node)
        return self

    def has_reduced_elementwise_nodes(self) -> bool:
        return not is_reduction_node(self.nodes_groups[0]) and len(self.reduced_args) > 0

    def dependent_nodes(self, keep_reduce_node: bool):
        node_map = {}
        reduce_nodes = []
        if not keep_reduce_node and self.has_reduced_elementwise_nodes():
            for item in self.nodes_groups:
                if not isinstance(item, NodeGroup):
                    node_map[item.name] = item
            return node_map, reduce_nodes
        for item in self.nodes_groups:
            if isinstance(item, NodeGroup):
                node_map.update(item.dependent_nodes(keep_reduce_node)[0])
            elif keep_reduce_node or not is_reduction_node(item):
                node_map[item.name] = item
            else:
                reduce_nodes.append(item)
        return node_map, reduce_nodes

    # finalize the group, and return the flatten nodes
    def flatten(self, sorted_nodes: list[NodeProto]) -> tuple[list[NodeProto], list[list[int]]]:
        if self.autotune_configs.requires_for_loop:
            layers = []
            group_layer = [self]
            while len(group_layer) > 0:
                node_map = {}
                reduce_node_map = {}
                next_layer = []
                for group in group_layer:
                    sub_node_map, reduce_nodes = group.dependent_nodes(False)
                    node_map.update(sub_node_map)
                    for node in reduce_nodes:
                        reduce_node_map[node.name] = node
                    next_layer.extend([item for item in group.nodes_groups if isinstance(item, NodeGroup)])
                layers.append((node_map, reduce_node_map.values()))
                group_layer = next_layer
            nodes = []
            layer_indices = []
            for i in range(len(layers) - 1, -1, -1):
                sub_nodes = list(layers[i][0].values())
                sub_nodes.sort(key=sorted_nodes.index)
                nodes.extend(sub_nodes)
                sub_layer_indices = []
                for node in layers[i][1]:
                    nodes.append(node)
                    sub_layer_indices.append(len(nodes) - 1)
                layer_indices.append(sub_layer_indices)
            return nodes, layer_indices
        node_map, _ = self.dependent_nodes(True)
        nodes = list(node_map.values())
        nodes.sort(key=sorted_nodes.index)
        return nodes, []

    def try_merge(self, other) -> bool:
        if (
            self.target_shape != other.target_shape
            or self.reduce_axes != other.reduce_axes
            or self.has_reduced_elementwise_nodes() != other.has_reduced_elementwise_nodes()
        ):
            return False
        self.nodes_groups.extend(other.nodes_groups)
        self.reduced_args.update(other.reduced_args)
        return True


class KernelIO:
    """
    Used to represent the inputs and outputs of a kernel(triton kernel).
    """

    def __init__(self):
        self.module_inputs: list[str] = []
        self.cross_kernel_inputs: list[str] = []
        self.constants: list[str] = []
        self.module_outputs: list[str] = []
        self.cross_kernel_outputs: list[str] = []
        self.internal_args: list[str] = []


class GraphLowering:
    """
    GraphLowering does manager all steps of lowering onnx graph to triton irnode.
    1. partition the graph into kernels (one or more kernels).
        Manager to allocate inputs, outputs and buffers reuse between kernels.
        we call it Module
    2. convert kernel to irnodes.
    3. analyze the IR relationship and buffer/Tensor inside a kernel.
    4. Generate the Auto-Tune configs for each kernel, Tunning speed/Running faster depends.

    we will end up getting a tree-liked IR structure with explicit input/output and intermediate buffer.

    """

    def __init__(self, sorted_graph: SortedGraph):
        self._sorted_graph: SortedGraph = sorted_graph
        self._node_arg_infos: dict[str, TensorInfo] = sorted_graph.node_arg_infos
        self._module_inputs: list[TensorArg] = []
        self._module_outputs: list[TensorArg] = []
        self._module_constants: list[TensorArg] = []
        self._module_input_names: set[str] = set()
        self._module_output_names: set[str] = set()
        self._module_constant_names: set[str] = set()
        self._tensor_args: dict[str, TensorArg] = {}
        # Extract module inputs, outputs and constants.
        self._extract_module_io()

        # Group nodes into NodeGroups, each NodeGroup represents a kernel.
        self._groups: list[NodeGroup] = []
        self._group_nodes()

        # Convert NodeGroups to KernelNodes.
        self._kernel_nodes: list[KernelNode] = []
        self._kernel_io_list: list[KernelIO] = []
        self._lower()

    # A module is map to a real onnx graph.
    def _extract_module_io(self):
        graph = self._sorted_graph.original_graph
        self._module_inputs = [TensorArg(input.name, self._node_arg_infos[input.name]) for input in graph.input]
        self._module_input_names = {arg.name for arg in self._module_inputs}
        self._module_outputs = [TensorArg(output.name, self._node_arg_infos[output.name]) for output in graph.output]
        self._module_output_names = {arg.name for arg in self._module_outputs}
        for initializer in graph.initializer:
            data = to_torch_tensor(initializer)
            self._module_constants.append(TensorArg(initializer.name, data=data))
        for const_node in self._sorted_graph.const_nodes:
            data = to_torch_tensor(const_node)
            self._module_constants.append(TensorArg(const_node.output[0], data=data))
        self._module_constant_names = {arg.name for arg in self._module_constants}
        self._tensor_args = {
            arg.name: arg for arg in itertools.chain(self._module_inputs, self._module_outputs, self._module_constants)
        }

    def _get_reduce_info(self, node) -> tuple[int, list[int]]:
        assert is_reduction_node(node)
        input_rank = len(self._node_arg_infos[node.input[0]].shape)
        return get_reduce_info(node, self._sorted_graph.original_graph, input_rank)

    def _process_node(self, node: NodeProto, precessors: dict[str, list[NodeProto]], group: NodeGroup):
        dependent_nodes = set()
        dependent_nodes.add(node.name)
        for precessor in precessors[node.name]:
            if precessor.name in dependent_nodes:
                continue
            keep_dims = 1
            reduce_axes = []
            if is_reduction_node(precessor):
                keep_dims, reduce_axes = self._get_reduce_info(precessor)
            split_if_different = any(
                output in self._sorted_graph.elementwise_graph_outputs for output in precessor.output
            )
            if group.compatible(precessor, reduce_axes, keep_dims, split_if_different):
                next_group = group.add_node(precessor, reduce_axes, keep_dims)
                dependent_nodes.update(self._process_node(precessor, precessors, next_group))
        return dependent_nodes

    def _group_nodes(self):
        producers = {}
        precessors = defaultdict(list)
        processed = set()
        groups = []
        sorted_nodes = self._sorted_graph.sorted_nodes
        for node in sorted_nodes:
            for output in node.output:
                producers[output] = node
            for input in node.input:
                if input in producers:
                    precessors[node.name].append(producers[input])
        for value in precessors.values():
            value.sort(key=sorted_nodes.index, reverse=True)
        for idx in range(len(sorted_nodes) - 1, -1, -1):
            node = sorted_nodes[idx]
            if node.name not in processed:
                reduce_axes = []
                keep_dims = 1
                if is_reduction_node(node):
                    keep_dims, reduce_axes = self._get_reduce_info(node)
                groups.append(NodeGroup(node, reduce_axes, keep_dims, self._node_arg_infos))
                processed.update(self._process_node(node, precessors, groups[-1]))

        # Merge groups with same target shape and reduce axes without dependency.
        group_dependencies = defaultdict(set)
        for i in range(len(groups) - 1):
            group_inputs = set()
            for node in groups[i].dependent_nodes(True)[0].values():
                group_inputs.update(node.input)
            for j in range(i + 1, len(groups)):
                if any(output in group_inputs for output in groups[j].nodes_groups[0].output):
                    group_dependencies[i].add(j)
                    for k in range(i):
                        if i in group_dependencies[k]:
                            group_dependencies[k].add(j)

        flag = set()
        for i, group_i in enumerate(groups):
            if i in flag:
                continue
            for j, group_j in enumerate(groups):
                if j <= i:
                    continue
                if j not in flag and j not in group_dependencies[i] and group_i.try_merge(group_j):
                    flag.add(j)
            self._groups.append(group_i)
            flag.add(i)

    def _get_node_io(self, node: NodeProto) -> tuple[list[TensorArg], list[TensorArg]]:
        input_args = []
        for input in node.input:
            if input in self._tensor_args:
                input_args.append(self._tensor_args[input])
            else:
                input_args.append(TensorArg(input, self._node_arg_infos[input]))
                self._tensor_args[input] = input_args[-1]
        output_args = []
        for output in node.output:
            if output in self._tensor_args:
                output_args.append(self._tensor_args[output])
            else:
                output_args.append(TensorArg(output, self._node_arg_infos[output]))
                self._tensor_args[output] = output_args[-1]
        return input_args, output_args

    def _extract_kernel_io(self, nodes: list[NodeProto]) -> KernelIO:
        kernel_io = KernelIO()
        input_set = set()
        output_set = set()
        for node in nodes:
            for input in node.input:
                if input in input_set:
                    continue
                elif input in self._module_constant_names:
                    kernel_io.constants.append(input)
                elif input in self._module_input_names:
                    kernel_io.module_inputs.append(input)
                elif input not in output_set:
                    kernel_io.cross_kernel_inputs.append(input)
                input_set.add(input)
            for output in node.output:
                if output in output_set:
                    continue
                if output in self._module_output_names:
                    kernel_io.module_outputs.append(output)
                else:
                    kernel_io.internal_args.append(output)
                output_set.add(output)
        return kernel_io

    def _to_compute_node(self, node: NodeProto, offset_calc: OffsetCalculator):
        inputs, outputs = self._get_node_io(node)
        op_type = node.op_type
        if op_type == "Dropout":
            return DropoutNode(inputs, outputs, offset_calc)
        if is_reduction_node(node):
            return ReduceNode(op_type, inputs, outputs, offset_calc)
        attributes = {}
        for attr in node.attribute:
            attributes[attr.name] = helper.get_attribute_value(attr)
        return ComputeNode(op_type, inputs, outputs, attributes)

    def _analyze_kernel_io_list(self):
        cross_kernel_inputs = set()
        for kernel_io in self._kernel_io_list:
            cross_kernel_inputs.update(kernel_io.cross_kernel_inputs)
        for kernel_io in self._kernel_io_list:
            kernel_io.cross_kernel_outputs = [arg for arg in kernel_io.internal_args if arg in cross_kernel_inputs]
            kernel_io.internal_args = [
                arg for arg in kernel_io.internal_args if arg not in kernel_io.cross_kernel_outputs
            ]

    def _insert_load_and_store(self, kernel_node: KernelNode):
        input_names = [input.name for input in kernel_node.inputs]
        output_name_map = {}
        for output in kernel_node.outputs:
            output_name_map[output.name] = 0
        for node in kernel_node.sub_nodes:
            for output in node.outputs:
                if output.name in output_name_map:
                    output_name_map[output.name] += 1
        sub_nodes = kernel_node.sub_nodes
        new_sub_nodes = []
        cur = 0
        nxt = 0
        reduce_store_nodes = []
        while True:
            while nxt < len(sub_nodes) and not isinstance(sub_nodes[nxt], ReduceForLoopEnd):
                nxt += 1
            load_cache = set()
            load_nodes = []
            store_nodes = []
            for idx in range(cur, nxt):
                for input in sub_nodes[idx].inputs:
                    if input.name in kernel_node.constants or input.name in input_names:
                        if (input.data is not None and input.data.numel() == 1) or input.name in load_cache:
                            continue
                        load_nodes.append(IONode(input, kernel_node.offset_calc, True))
                        load_cache.add(input.name)
                for output in sub_nodes[idx].outputs:
                    if output.name in output_name_map:
                        output_name_map[output.name] -= 1
                        if output_name_map[output.name] == 0:
                            store_nodes.append(IONode(output, kernel_node.offset_calc, False))
            if isinstance(sub_nodes[cur], ReduceForLoopStart):
                new_sub_nodes.append(sub_nodes[cur])
                cur += 1
            if nxt < len(sub_nodes):
                assert isinstance(sub_nodes[nxt], ReduceForLoopEnd)
                for reduce_node in sub_nodes[nxt].reduce_nodes:
                    input = reduce_node.inputs[0]
                    if input.name in kernel_node.constants or input.name in input_names:
                        if (input.data is not None and input.data.numel() == 1) or input.name in load_cache:
                            continue
                        load_nodes.append(IONode(input, kernel_node.offset_calc, True))
                        load_cache.add(input.name)
            new_sub_nodes.extend(load_nodes)
            new_sub_nodes.extend(sub_nodes[cur:nxt])
            new_sub_nodes.extend(store_nodes)
            if nxt < len(sub_nodes):
                assert isinstance(sub_nodes[nxt], ReduceForLoopEnd)
                for reduce_node in sub_nodes[nxt].reduce_nodes:
                    if reduce_node.outputs[0].name in output_name_map:
                        reduce_store_nodes.append(IONode(reduce_node.outputs[0], kernel_node.offset_calc, False))
                new_sub_nodes.append(sub_nodes[nxt])
                nxt += 1
            cur = nxt
            if cur >= len(sub_nodes):
                break
        new_sub_nodes.extend(reduce_store_nodes)
        kernel_node.sub_nodes = new_sub_nodes

    def _lower(self):
        for group in self._groups:
            is_reduction_kernel = len(group.reduce_axes) > 0
            target_shape = group.target_shape
            # The inputs and outputs will be initialized later.
            kernel_node = (
                ReduceKernelNode([], [], target_shape, group.reduce_axes, group.reduced_args)
                if is_reduction_kernel
                else ElementwiseKernelNode([], [], target_shape)
            )
            self._kernel_nodes.append(kernel_node)
            sub_nodes = []
            nodes, layer_indices = group.flatten(self._sorted_graph.sorted_nodes)
            self._kernel_io_list.append(self._extract_kernel_io(nodes))
            if group.autotune_configs.requires_for_loop:
                start = 0
                for layer_idx, indices in enumerate(layer_indices):
                    need_for_loop = True
                    if layer_idx == len(layer_indices) - 1 and group.has_reduced_elementwise_nodes():
                        assert len(indices) == 0
                        need_for_loop = False
                    reduce_nodes = [self._to_compute_node(nodes[idx], kernel_node.offset_calc) for idx in indices]
                    assert all(isinstance(node, ReduceNode) for node in reduce_nodes)
                    if need_for_loop:
                        sub_nodes.append(ReduceForLoopStart(reduce_nodes, kernel_node.offset_calc))
                    end = indices[0] if len(indices) > 0 else len(nodes)
                    for idx in range(start, end):
                        node = nodes[idx]
                        assert not is_reduction_node(node)
                        sub_nodes.append(self._to_compute_node(node, kernel_node.offset_calc))
                        if node.op_type == "Dropout":
                            self._kernel_nodes[-1].has_dropout = True
                    if len(indices) > 0:
                        sub_nodes.append(ReduceForLoopEnd(reduce_nodes, kernel_node.offset_calc))
                    start = indices[len(indices) - 1] + 1 if len(indices) > 0 else len(nodes)
            else:
                for node in nodes:
                    sub_nodes.append(self._to_compute_node(node, kernel_node.offset_calc))
                    if node.op_type == "Dropout":
                        self._kernel_nodes[-1].has_dropout = True
            self._kernel_nodes[-1].sub_nodes = sub_nodes

        if any(kernel_node.has_dropout for kernel_node in self._kernel_nodes):
            warnings.warn("Use triton's random for Dropout, ignore the random seed from ORT.", UserWarning)

        self._analyze_kernel_io_list()
        cross_kernel_arg_map = {}
        for idx, kernel_io in enumerate(self._kernel_io_list):
            for output in itertools.chain(kernel_io.cross_kernel_outputs, kernel_io.module_outputs):
                cross_kernel_arg_map[output] = idx
        dependency = defaultdict(set)
        for idx, kernel_io in enumerate(self._kernel_io_list):
            for input in kernel_io.cross_kernel_inputs:
                dependency[cross_kernel_arg_map[input]].add(idx)
        visited = set()
        sorted_indices = []

        def _topological_sort_internal(idx):
            visited.add(idx)
            for next_idx in dependency[idx]:
                if next_idx not in visited:
                    _topological_sort_internal(next_idx)
            sorted_indices.insert(0, idx)

        for idx in range(len(self._kernel_io_list)):
            if idx not in visited:
                _topological_sort_internal(idx)

        self._kernel_nodes = [self._kernel_nodes[idx] for idx in sorted_indices]
        self._kernel_io_list = [self._kernel_io_list[idx] for idx in sorted_indices]
        cross_kernel_arg_map.clear()
        for idx, kernel_io in enumerate(self._kernel_io_list):
            for arg in kernel_io.cross_kernel_inputs:
                if arg not in self._module_output_names:
                    cross_kernel_arg_map[arg] = idx

        self._cross_kernel_args = [(self._tensor_args[key], value) for key, value in cross_kernel_arg_map.items()]

        for idx, kernel_node in enumerate(self._kernel_nodes):
            kernel_io = self._kernel_io_list[idx]
            kernel_node.internal_args.update(kernel_io.internal_args)
            kernel_node.inputs = [
                self._tensor_args[name]
                for name in itertools.chain(kernel_io.module_inputs, kernel_io.cross_kernel_inputs)
            ]
            kernel_node.outputs = [
                self._tensor_args[name]
                for name in itertools.chain(kernel_io.module_outputs, kernel_io.cross_kernel_outputs)
            ]
            for name in kernel_io.constants:
                kernel_node.constants[name] = self._tensor_args[name]
            self._insert_load_and_store(kernel_node)
            kernel_node.gen_variable_names()

    def module_node(self, func_name: str):
        return ModuleNode(
            func_name,
            self._module_inputs,
            self._module_outputs,
            self._module_constants,
            self._cross_kernel_args,
            self._kernel_nodes,
        )


def lower(func_name: str, sorted_graph: SortedGraph) -> ModuleNode:
    return GraphLowering(sorted_graph).module_node(func_name)



#!/usr/bin/env python3
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

import argparse
import os
import shlex
import shutil
import sys
from pathlib import Path

from logger import get_logger

SCRIPT_DIR = os.path.dirname(os.path.realpath(__file__))
REPO_DIR = os.path.normpath(os.path.join(SCRIPT_DIR, "..", ".."))
sys.path.append(os.path.join(REPO_DIR, "tools", "python"))
from util import run  # noqa: E402

log = get_logger("get_docker_image")


def parse_args():
    parser = argparse.ArgumentParser(
        description="Build a docker image and push it to a remote Azure Container Registry."
        "The content in the remote registry can be used as a cache when we need to build the thing again."
        "The user must be logged in to the container registry."
    )

    parser.add_argument("--dockerfile", default="Dockerfile", help="Path to the Dockerfile.")
    parser.add_argument("--context", default=".", help="Path to the build context.")
    parser.add_argument(
        "--docker-build-args", default="", help="Arguments that will be passed to the 'docker build' command."
    )

    parser.add_argument(
        "--container-registry",
        help="The Azure container registry name. If not provided, no container registry will be used.",
    )
    parser.add_argument("--repository", required=True, help="The image repository name.")

    parser.add_argument("--use_imagecache", action="store_true", help="use cached image in pipeline cache")

    parser.add_argument("--docker-path", default="docker", help="Path to docker.")

    parser.add_argument("--manylinux-src", default="manylinux", help="Path to manylinux src folder")

    parser.add_argument(
        "--multiple_repos",
        action="store_true",
        help="used in packaging pipeline, which couldn't use get-docker-images-steps.yml",
    )

    return parser.parse_args()


def main():
    args = parse_args()

    log.debug(f"Dockerfile: {args.dockerfile}, context: {args.context}, docker build args: '{args.docker_build_args}'")

    use_container_registry = args.container_registry is not None

    if not use_container_registry:
        log.info("No container registry will be used")

    full_image_name = (
        f"{args.container_registry}.azurecr.io/{args.repository}:latest"
        if use_container_registry
        else f"{args.repository}:latest"
    )

    log.info(f"Image: {full_image_name}")

    dst_deps_file = Path(args.context) / "scripts" / "deps.txt"
    # The docker file may provide a special deps.txt in its docker context dir and uses that one.
    # Otherwise, copy a generic one from this repo's cmake dir.
    if not dst_deps_file.exists():
        log.info(f"Copy deps.txt to : {dst_deps_file}")
        shutil.copyfile(Path(REPO_DIR) / "cmake" / "deps.txt", str(dst_deps_file))

    if "manylinux" in args.dockerfile and args.multiple_repos:
        manylinux_build_scripts_folder = Path(args.manylinux_src) / "docker" / "build_scripts"
        dest = Path(args.context) / "build_scripts"
        if dest.exists():
            log.info(f"Deleting: {dest!s}")
            shutil.rmtree(str(dest))
        shutil.copytree(str(manylinux_build_scripts_folder), str(dest))
        src_entrypoint_file = str(Path(args.manylinux_src) / "docker" / "manylinux-entrypoint")
        dst_entrypoint_file = str(Path(args.context) / "manylinux-entrypoint")
        shutil.copyfile(src_entrypoint_file, dst_entrypoint_file)
        shutil.copymode(src_entrypoint_file, dst_entrypoint_file)
        run(
            "patch",
            "-p1",
            "-i",
            str((Path(SCRIPT_DIR) / "github" / "linux" / "docker" / "manylinux.patch").resolve()),
            cwd=str(dest),
        )

    if use_container_registry:
        run(
            args.docker_path,
            "--log-level",
            "error",
            "buildx",
            "build",
            "--load",
            "--tag",
            full_image_name,
            "--cache-from=type=registry,ref=" + full_image_name,
            "--build-arg",
            "BUILDKIT_INLINE_CACHE=1",
            *shlex.split(args.docker_build_args),
            "-f",
            args.dockerfile,
            args.context,
        )
        run(
            args.docker_path,
            "push",
            full_image_name,
        )
    else:
        log.info("Building image...")
        run(
            args.docker_path,
            "build",
            "--pull",
            *shlex.split(args.docker_build_args),
            "--tag",
            full_image_name,
            "--file",
            args.dockerfile,
            args.context,
        )

    # tag so we can refer to the image by repository name
    run(args.docker_path, "tag", full_image_name, args.repository)

    return 0


if __name__ == "__main__":
    sys.exit(main())



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

import argparse
import contextlib
import os
import sys
import typing

# the import of FbsTypeInfo sets up the path so we can import ort_flatbuffers_py
from util.ort_format_model.types import FbsTypeInfo  # isort:skip
import ort_flatbuffers_py.fbs as fbs  # isort:skip


class OrtFormatModelDumper:
    "Class to dump an ORT format model."

    def __init__(self, model_path: str):
        """
        Initialize ORT format model dumper
        :param model_path: Path to model
        """
        self._file = open(model_path, "rb").read()  # noqa: SIM115
        self._buffer = bytearray(self._file)
        if not fbs.InferenceSession.InferenceSession.InferenceSessionBufferHasIdentifier(self._buffer, 0):
            raise RuntimeError(f"File does not appear to be a valid ORT format model: '{model_path}'")
        self._inference_session = fbs.InferenceSession.InferenceSession.GetRootAsInferenceSession(self._buffer, 0)
        self._model = self._inference_session.Model()

    def _dump_initializers(self, graph: fbs.Graph):
        print("Initializers:")
        for idx in range(graph.InitializersLength()):
            tensor = graph.Initializers(idx)
            dims = []
            for dim in range(tensor.DimsLength()):
                dims.append(tensor.Dims(dim))

            print(f"{tensor.Name().decode()} data_type={tensor.DataType()} dims={dims}")
        print("--------")

    def _dump_nodeargs(self, graph: fbs.Graph):
        print("NodeArgs:")
        for idx in range(graph.NodeArgsLength()):
            node_arg = graph.NodeArgs(idx)
            type = node_arg.Type()
            if not type:
                # NodeArg for optional value that does not exist
                continue

            type_str = FbsTypeInfo.typeinfo_to_str(type)
            value_type = type.ValueType()
            value = type.Value()
            dims = None
            if value_type == fbs.TypeInfoValue.TypeInfoValue.tensor_type:
                tensor_type_and_shape = fbs.TensorTypeAndShape.TensorTypeAndShape()
                tensor_type_and_shape.Init(value.Bytes, value.Pos)
                shape = tensor_type_and_shape.Shape()
                if shape:
                    dims = []
                    for dim in range(shape.DimLength()):
                        d = shape.Dim(dim).Value()
                        if d.DimType() == fbs.DimensionValueType.DimensionValueType.VALUE:
                            dims.append(str(d.DimValue()))
                        elif d.DimType() == fbs.DimensionValueType.DimensionValueType.PARAM:
                            dims.append(d.DimParam().decode())
                        else:
                            dims.append("?")
            else:
                dims = None

            print(f"{node_arg.Name().decode()} type={type_str} dims={dims}")
        print("--------")

    def _dump_node(self, node: fbs.Node):
        optype = node.OpType().decode()
        domain = node.Domain().decode() or "ai.onnx"  # empty domain defaults to ai.onnx
        since_version = node.SinceVersion()

        inputs = [node.Inputs(i).decode() for i in range(node.InputsLength())]
        outputs = [node.Outputs(i).decode() for i in range(node.OutputsLength())]
        print(
            f"{node.Index()}:{node.Name().decode()}({domain}:{optype}:{since_version}) "
            f"inputs=[{','.join(inputs)}] outputs=[{','.join(outputs)}]"
        )

    def _dump_graph(self, graph: fbs.Graph):
        """
        Process one level of the Graph, descending into any subgraphs when they are found
        """

        self._dump_initializers(graph)
        self._dump_nodeargs(graph)
        print("Nodes:")
        for i in range(graph.NodesLength()):
            node = graph.Nodes(i)
            self._dump_node(node)

            # Read all the attributes
            for j in range(node.AttributesLength()):
                attr = node.Attributes(j)
                attr_type = attr.Type()
                if attr_type == fbs.AttributeType.AttributeType.GRAPH:
                    print(f"## Subgraph for {node.OpType().decode()}.{attr.Name().decode()} ##")
                    self._dump_graph(attr.G())
                    print(f"## End {node.OpType().decode()}.{attr.Name().decode()} Subgraph ##")
                elif attr_type == fbs.AttributeType.AttributeType.GRAPHS:
                    # the ONNX spec doesn't currently define any operators that have multiple graphs in an attribute
                    # so entering this 'elif' isn't currently possible
                    print(f"## Subgraphs for {node.OpType().decode()}.{attr.Name().decode()} ##")
                    for k in range(attr.GraphsLength()):
                        print(f"## Subgraph {k} ##")
                        self._dump_graph(attr.Graphs(k))
                        print(f"## End Subgraph {k} ##")

    def dump(self, output: typing.IO):
        with contextlib.redirect_stdout(output):
            print(f"ORT format version: {self._inference_session.OrtVersion().decode()}")
            print("--------")

            graph = self._model.Graph()
            self._dump_graph(graph)


def parse_args():
    parser = argparse.ArgumentParser(
        os.path.basename(__file__), description="Dump an ORT format model. Output is to <model_path>.txt"
    )
    parser.add_argument("--stdout", action="store_true", help="Dump to stdout instead of writing to file.")
    parser.add_argument("model_path", help="Path to ORT format model")
    args = parser.parse_args()

    if not os.path.isfile(args.model_path):
        parser.error(f"{args.model_path} is not a file.")

    return args


def main():
    args = parse_args()
    d = OrtFormatModelDumper(args.model_path)

    if args.stdout:
        d.dump(sys.stdout)
    else:
        output_filename = args.model_path + ".txt"
        with open(output_filename, "w", encoding="utf-8") as ofile:
            d.dump(ofile)


if __name__ == "__main__":
    main()



import argparse
import os
import sys
import time
from pathlib import Path

import requests

script_description = """
After building ONNXRuntime for Android or iOS, use this script to upload the app and test files to BrowserStack then
run the tests on the specified devices.

Find the Android test app in the repo here (as of rel-1.19.2):
java/src/test/android
"""


def response_to_json(response):
    response.raise_for_status()
    response_json = response.json()
    print(response_json)
    print("-" * 30)
    return response_json


def upload_apk_parse_json(post_url, apk_path, id, token):
    with open(apk_path, "rb") as apk_file:
        response = requests.post(post_url, files={"file": apk_file}, auth=(id, token), timeout=180)
    return response_to_json(response)


def browserstack_build_request(devices, app_url, test_suite_url, test_platform, id, token, project, build_tag):
    headers = {}

    json_data = {
        "devices": devices,
        "app": app_url,
        "testSuite": test_suite_url,
        "project": project,
        "buildTag": build_tag,
        "deviceLogs": True,
    }

    build_response = requests.post(
        f"https://api-cloud.browserstack.com/app-automate/{test_platform}/v2/build",
        headers=headers,
        json=json_data,
        auth=(id, token),
        timeout=180,
    )

    return response_to_json(build_response)


def build_query_loop(build_id, test_platform, id, token):
    """Called after a build is initiated on Browserstack. Repeatedly queries the REST endpoint to check for the
    status of the tests in 30 second intervals.
    """
    tests_status = "running"

    while tests_status == "running":
        time.sleep(30)

        test_response = requests.get(
            f"https://api-cloud.browserstack.com/app-automate/{test_platform}/v2/builds/{build_id}",
            auth=(id, token),
            timeout=30,
        )

        test_response_json = response_to_json(test_response)
        tests_status = test_response_json["status"]

    return tests_status


if __name__ == "__main__":
    # handle cli args
    parser = argparse.ArgumentParser(script_description)

    parser.add_argument(
        "--test_platform", type=str, help="Testing platform", choices=["espresso", "xcuitest"], required=True
    )
    parser.add_argument(
        "--app_path",
        type=Path,
        help=(
            "Path to the app file. "
            "For Android, typically, the app file (the APK) is in "
            "{build_output_dir}/android_test/android/app/build/outputs/apk/debug/app-debug.apk"
            ". For iOS, you will have to build an IPA file from the test app, which is built from the .xcarchive path"
        ),
        required=True,
    )
    parser.add_argument(
        "--test_path",
        type=Path,
        help=(
            "Path to the test suite file. "
            "Typically, the test APK is in "
            "{build_output_dir}/android_test/android/app/build/outputs/apk/androidTest/debug/app-debug-androidTest.apk"
            ". For iOS, you will have to create a .zip of the tests. After manually building the tests, the tests that you need to zip will be in {{Xcode DerivedData Folder Path}}/Build/Products"
        ),
        required=True,
    )
    parser.add_argument(
        "--devices",
        type=str,
        nargs="+",
        help="List of devices to run the tests on. For more info, "
        "see https://www.browserstack.com/docs/app-automate/espresso/specify-devices (Android) or https://www.browserstack.com/docs/app-automate/xcuitest/specify-devices (iOS)",
        required=True,
    )

    parser.add_argument(
        "--project",
        type=str,
        help="Identifier to logically group multiple builds together",
        default="ONNXRuntime tests",
    )
    parser.add_argument("--build_tag", type=str, help="Identifier to tag the build with a unique name", default="")
    args = parser.parse_args()

    try:
        browserstack_id = os.environ["BROWSERSTACK_ID"]
        browserstack_token = os.environ["BROWSERSTACK_TOKEN"]
    except KeyError:
        print("Please set the environment variables BROWSERSTACK_ID and BROWSERSTACK_TOKEN")
        print(
            "These values will be found at https://app-automate.browserstack.com/dashboard/v2 & clicking 'ACCESS KEY'"
        )
        sys.exit(1)

    # Upload the app and test suites
    upload_app_json = upload_apk_parse_json(
        f"https://api-cloud.browserstack.com/app-automate/{args.test_platform}/v2/app",
        args.app_path,
        browserstack_id,
        browserstack_token,
    )
    upload_test_json = upload_apk_parse_json(
        f"https://api-cloud.browserstack.com/app-automate/{args.test_platform}/v2/test-suite",
        args.test_path,
        browserstack_id,
        browserstack_token,
    )

    # Initiate build (send request to run the tests)
    build_response_json = browserstack_build_request(
        args.devices,
        upload_app_json["app_url"],
        upload_test_json["test_suite_url"],
        args.test_platform,
        browserstack_id,
        browserstack_token,
        args.project,
        args.build_tag,
    )

    # Get build status until the tests are no longer running
    tests_status = build_query_loop(
        build_response_json["build_id"], args.test_platform, browserstack_id, browserstack_token
    )

    test_suite_details_url = (
        f"https://app-automate.browserstack.com/dashboard/v2/builds/{build_response_json['build_id']}"
    )

    print("=" * 30)
    print("Test suite details: ", test_suite_details_url)
    print("=" * 30)
    if tests_status != "passed":
        output_str = f"""Tests failed. Go to {test_suite_details_url} for the BrowserStack run.

        For more details on how to access the BrowserStack logs, go to the following page in the team OneNote:
        ONNX Ecosystem Team Notebook -> ORT Mobile -> End-to-end tests for iOS & Android
        """

        raise Exception(output_str)



import onnx
from onnx import TensorProto, helper

if_body = helper.make_graph(
    [
        # need to use main_graph_initializer in a way that can't be constant folded
        helper.make_node("Add", ["state_var_in", "main_graph_initializer"], ["add_out"], "If_add"),
        helper.make_node("Cast", ["add_out"], ["output"], to=TensorProto.BOOL),
    ],
    "if_branch_body",
    [
        # no explicit inputs
    ],
    [
        helper.make_tensor_value_info("output", TensorProto.BOOL, [1]),  # how is this getting a type of float?
    ],
)

# Loop body graph with If node and usage of main_graph_initializer on this level
body = helper.make_graph(
    [
        # Add node that can be constant folded. Creates NodeArg when created but that implicit usage of an outer scope
        # value main_graph_initializer goes away after constant folding
        helper.make_node(
            "Add",
            ["sub_graph_initializer", "main_graph_initializer"],
            ["initializer_sum"],
            "Add1",
        ),
        helper.make_node("Add", ["initializer_sum", "loop_state_in"], ["loop_state_out"], "Add2"),
        # If node to create usage of main_graph_initializer another level down
        helper.make_node(
            "If",
            ["subgraph_keep_going_in"],
            ["subgraph_keep_going_out"],
            "If1",
            then_branch=if_body,
            else_branch=if_body,
        ),
    ],
    "Loop_body",
    [
        helper.make_tensor_value_info("iteration_num", TensorProto.INT64, [1]),
        helper.make_tensor_value_info("subgraph_keep_going_in", TensorProto.BOOL, [1]),
        helper.make_tensor_value_info("loop_state_in", TensorProto.FLOAT, [1]),
    ],
    [
        helper.make_tensor_value_info("subgraph_keep_going_out", TensorProto.BOOL, [1]),
        helper.make_tensor_value_info("loop_state_out", TensorProto.FLOAT, [1]),
    ],
    [helper.make_tensor("sub_graph_initializer", TensorProto.FLOAT, [1], [1.0])],
)

# Create the main graph
graph_proto = helper.make_graph(
    [
        helper.make_node(
            "Loop",
            ["max_trip_count", "keep_going", "state_var_in"],
            ["state_var_out"],
            "Loop1",
            body=body,
        )
    ],
    "Main_graph",
    [
        helper.make_tensor_value_info("state_var_in", TensorProto.FLOAT, [1]),
    ],
    [
        helper.make_tensor_value_info("state_var_out", TensorProto.FLOAT, [1]),
    ],
    [
        helper.make_tensor("max_trip_count", TensorProto.INT64, [1], [1]),
        helper.make_tensor("main_graph_initializer", TensorProto.FLOAT, [1], [1.0]),
        helper.make_tensor("keep_going", TensorProto.BOOL, [1], [True]),
    ],
)

model = helper.make_model(graph_proto)
onnx.save(model, "ort_github_issue_4031.onnx")



#!/usr/bin/env python
# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import unittest

import numpy as np
import onnx
from onnx import TensorProto, helper
from op_test_utils import (
    TestDataFeeds,
    check_model_correctness,
    check_op_nodes,
    check_op_type_count,
    check_qtype_by_node_type,
)

from onnxruntime.quantization import QuantFormat, QuantType, quantize_static


class TestOpResize(unittest.TestCase):
    def input_feeds(self, n, name2shape):
        input_data_list = []
        for _i in range(n):
            inputs = {}
            for name, shape in name2shape.items():
                inputs.update({name: np.random.randint(-1, 2, shape).astype(np.float32)})
            input_data_list.extend([inputs])
        dr = TestDataFeeds(input_data_list)
        return dr

    def construct_model_conv_resize(
        self,
        output_model_path,
        conv_input_shape,
        conv_weight_shape,
        resize_input_shape,
        resize_output_shape,
        resize_attrs,
        resize_roi,
        resize_scales,
        resize_sizes,
    ):
        #      (input)
        #          \
        #         Conv
        #        /    \
        #   Identity   Resize
        #    /            \
        # (identity_out)  (output)
        input_tensor = helper.make_tensor_value_info("input", TensorProto.FLOAT, conv_input_shape)

        conv_weight_arr = np.random.randint(-1, 2, conv_weight_shape).astype(np.float32)
        conv_weight_initializer = onnx.numpy_helper.from_array(conv_weight_arr, name="conv1_weight")
        conv_node = onnx.helper.make_node("Conv", ["input", "conv1_weight"], ["conv_output"], name="conv_node")

        identity_out = helper.make_tensor_value_info("identity_out", TensorProto.FLOAT, resize_input_shape)
        identity_node = helper.make_node("Identity", ["conv_output"], ["identity_out"], name="IdentityNode")

        initializers = [conv_weight_initializer]

        output_tensor = helper.make_tensor_value_info("output", TensorProto.FLOAT, resize_output_shape)
        resize_inputs = ["conv_output"]  # resize_roi_name, resize_scales_name, resize_sizes_name]
        resize_node = helper.make_node("Resize", resize_inputs, ["output"], name="resize_node", **resize_attrs)

        if resize_roi is not None:
            resize_roi_name = "resize_roi"
            resize_roi_initializer = helper.make_tensor(
                resize_roi_name, TensorProto.FLOAT, [len(resize_roi)], resize_roi
            )
            initializers.extend([resize_roi_initializer])
            resize_node.input.extend([resize_roi_name])
        else:
            resize_node.input.extend([""])

        if resize_scales is not None:
            resize_scales_name = "resize_scales"
            resize_scales_initializer = helper.make_tensor(
                resize_scales_name,
                TensorProto.FLOAT,
                [len(resize_scales)],
                resize_scales,
            )
            initializers.extend([resize_scales_initializer])
            resize_node.input.extend([resize_scales_name])
        else:
            resize_node.input.extend([""])

        if resize_sizes is not None:
            resize_sizes_name = "resize_sizes"
            resize_sizes_initializer = helper.make_tensor(
                resize_sizes_name, TensorProto.INT64, [len(resize_sizes)], resize_sizes
            )
            initializers.extend([resize_sizes_initializer])
            resize_node.input.extend([resize_sizes_name])

        graph = helper.make_graph(
            [conv_node, identity_node, resize_node],
            "TestOpQuantizerResize_test_model",
            [input_tensor],
            [identity_out, output_tensor],
            initializer=initializers,
        )
        model = helper.make_model(graph, opset_imports=[helper.make_opsetid("", 13)])
        model.ir_version = 7  # use stable onnx ir version
        onnx.save(model, output_model_path)

    def quantize_resize_test(self, activation_type, weight_type, extra_options={}):  # noqa: B006
        np.random.seed(1)
        model_fp32_path = "resize_fp32.onnx"

        kwargs = {
            "coordinate_transformation_mode": "asymmetric",
            "mode": "nearest",
            "nearest_mode": "floor",
        }
        self.construct_model_conv_resize(
            model_fp32_path,
            [1, 2, 26, 42],
            [3, 2, 3, 3],
            [1, 3, 24, 40],
            [1, 3, 48, 80],
            kwargs,
            [0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, 1.0, 2.0, 2.0],
            None,
        )

        activation_proto_qtype = TensorProto.UINT8 if activation_type == QuantType.QUInt8 else TensorProto.INT8
        activation_type_str = "u8" if (activation_type == QuantType.QUInt8) else "s8"
        weight_type_str = "u8" if (weight_type == QuantType.QUInt8) else "s8"
        model_uint8_path = f"resize_{activation_type_str}{weight_type_str}.onnx"
        model_uint8_qdq_path = f"resize_{activation_type_str}{weight_type_str}_qdq.onnx"

        # Verify QOperator mode
        data_reader = self.input_feeds(1, {"input": [1, 2, 26, 42]})
        quantize_static(
            model_fp32_path,
            model_uint8_path,
            data_reader,
            quant_format=QuantFormat.QOperator,
            activation_type=activation_type,
            weight_type=weight_type,
            extra_options=extra_options,
        )
        # make sure resize become xint8 operator, its input name could tell that
        check_op_nodes(
            self,
            model_uint8_path,
            lambda node: (node.name != "resize_node" or node.input[0] != "conv_output"),
        )
        qnode_counts = {
            "QLinearConv": 1,
            "QuantizeLinear": 1,
            "DequantizeLinear": 2,
            "Resize": 1,
        }
        check_op_type_count(self, model_uint8_path, **qnode_counts)
        qnode_io_qtypes = {
            "QuantizeLinear": [
                ["i", 2, activation_proto_qtype],
                ["o", 0, activation_proto_qtype],
            ]
        }
        qnode_io_qtypes.update({"DequantizeLinear": [["i", 2, activation_proto_qtype]]})
        check_qtype_by_node_type(self, model_uint8_path, qnode_io_qtypes)
        data_reader.rewind()
        check_model_correctness(self, model_fp32_path, model_uint8_path, data_reader.get_next())

        # Verify QDQ mode
        data_reader.rewind()
        quantize_static(
            model_fp32_path,
            model_uint8_qdq_path,
            data_reader,
            quant_format=QuantFormat.QDQ,
            activation_type=activation_type,
            weight_type=weight_type,
            extra_options=extra_options,
        )
        qdqnode_counts = {
            "Conv": 1,
            "QuantizeLinear": 3,
            "DequantizeLinear": 4,
            "Resize": 1,
        }
        check_op_type_count(self, model_uint8_qdq_path, **qdqnode_counts)
        qnode_io_qtypes = {
            "QuantizeLinear": [
                ["i", 2, activation_proto_qtype],
                ["o", 0, activation_proto_qtype],
            ]
        }
        check_qtype_by_node_type(self, model_uint8_qdq_path, qnode_io_qtypes)
        data_reader.rewind()
        check_model_correctness(self, model_fp32_path, model_uint8_qdq_path, data_reader.get_next())

    def test_quantize_resize(self):
        self.quantize_resize_test(QuantType.QUInt8, QuantType.QUInt8)

    # TODO: Uncomment following after resize s8 support is enabled
    # def test_quantize_resize_s8s8(self):
    #     self.quantize_resize_test(QuantType.QInt8, QuantType.QInt8, extra_options = {'ActivationSymmetric' : True})


if __name__ == "__main__":
    unittest.main()



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
# pylint: disable=C0115,W0212,C0103,C0114


# -*- coding: UTF-8 -*-
import unittest

import numpy as np
from helper import get_name

import onnxruntime as onnxrt
from onnxruntime.capi.onnxruntime_pybind11_state import OrtValueVector, RunOptions


class TestSparseToDenseMatmul(unittest.TestCase):
    def test_run_sparse_output_ort_value_vector(self):
        """
        Try running models using the new run_with_ort_values
        sparse_initializer_as_output.onnx - requires no inputs, but only one output
        that comes from the initializer
        """
        # The below values are a part of the model
        sess = onnxrt.InferenceSession(
            get_name("sparse_initializer_as_output.onnx"),
            providers=onnxrt.get_available_providers(),
        )
        res = sess._sess.run_with_ort_values({}, ["values"], RunOptions())
        self.assertIsInstance(res, OrtValueVector)

    def test_run_sparse_output_only(self):
        """
        Try running models using the new run_with_ort_values
        sparse_initializer_as_output.onnx - requires no inputs, but only one output
        that comes from the initializer
        """
        # The below values are a part of the model
        dense_shape = [3, 3]
        values = np.array([1.764052391052246, 0.40015721321105957, 0.978738009929657], np.float32)
        indices = np.array([2, 3, 5], np.int64)
        sess = onnxrt.InferenceSession(
            get_name("sparse_initializer_as_output.onnx"),
            providers=onnxrt.get_available_providers(),
        )
        res = sess.run_with_ort_values(["values"], {})
        self.assertEqual(len(res), 1)
        ort_value = res[0]
        self.assertTrue(isinstance(ort_value, onnxrt.OrtValue))
        sparse_output = ort_value.as_sparse_tensor()
        self.assertTrue(isinstance(sparse_output, onnxrt.SparseTensor))
        self.assertEqual(dense_shape, sparse_output.dense_shape())
        self.assertTrue(np.array_equal(values, sparse_output.values()))
        self.assertTrue(np.array_equal(indices, sparse_output.as_coo_view().indices()))

    def test_run_contrib_sparse_mat_mul(self):
        """
        Multiple sparse COO tensor to dense
        """
        common_shape = [9, 9]  # inputs and oputputs same shape
        A_values = np.array(  # noqa: N806
            [
                1.0,
                2.0,
                3.0,
                4.0,
                5.0,
                6.0,
                7.0,
                8.0,
                9.0,
                10.0,
                11.0,
                12.0,
                13.0,
                14.0,
                15.0,
                16.0,
                17.0,
                18.0,
                19.0,
                20.0,
                21.0,
                22.0,
                23.0,
                24.0,
                25.0,
                26.0,
                27.0,
                28.0,
                29.0,
                30.0,
                31.0,
                32.0,
                33.0,
                34.0,
                35.0,
                36.0,
                37.0,
                38.0,
                39.0,
                40.0,
                41.0,
                42.0,
                43.0,
                44.0,
                45.0,
                46.0,
                47.0,
                48.0,
                49.0,
                50.0,
                51.0,
                52.0,
                53.0,
            ],
            np.float32,
        )
        # 2-D index
        A_indices = np.array(  # noqa: N806
            [
                0,
                1,
                0,
                2,
                0,
                6,
                0,
                7,
                0,
                8,
                1,
                0,
                1,
                1,
                1,
                2,
                1,
                6,
                1,
                7,
                1,
                8,
                2,
                0,
                2,
                1,
                2,
                2,
                2,
                6,
                2,
                7,
                2,
                8,
                3,
                3,
                3,
                4,
                3,
                5,
                3,
                6,
                3,
                7,
                3,
                8,
                4,
                3,
                4,
                4,
                4,
                5,
                4,
                6,
                4,
                7,
                4,
                8,
                5,
                3,
                5,
                4,
                5,
                5,
                5,
                6,
                5,
                7,
                5,
                8,
                6,
                0,
                6,
                1,
                6,
                2,
                6,
                3,
                6,
                4,
                6,
                5,
                7,
                0,
                7,
                1,
                7,
                2,
                7,
                3,
                7,
                4,
                7,
                5,
                8,
                0,
                8,
                1,
                8,
                2,
                8,
                3,
                8,
                4,
                8,
                5,
            ],
            np.int64,
        ).reshape((len(A_values), 2))

        cpu_device = onnxrt.OrtDevice.make("cpu", 0)
        sparse_tensor = onnxrt.SparseTensor.sparse_coo_from_numpy(common_shape, A_values, A_indices, cpu_device)
        A_ort_value = onnxrt.OrtValue.ort_value_from_sparse_tensor(sparse_tensor)  # noqa: N806

        B_data = np.array(  # noqa: N806
            [
                0,
                1,
                2,
                0,
                0,
                0,
                3,
                4,
                5,
                6,
                7,
                8,
                0,
                0,
                0,
                9,
                10,
                11,
                12,
                13,
                14,
                0,
                0,
                0,
                15,
                16,
                17,
                0,
                0,
                0,
                18,
                19,
                20,
                21,
                22,
                23,
                0,
                0,
                0,
                24,
                25,
                26,
                27,
                28,
                29,
                0,
                0,
                0,
                30,
                31,
                32,
                33,
                34,
                35,
                36,
                37,
                38,
                39,
                40,
                41,
                0,
                0,
                0,
                42,
                43,
                44,
                45,
                46,
                47,
                0,
                0,
                0,
                48,
                49,
                50,
                51,
                52,
                53,
                0,
                0,
                0,
            ],
            np.float32,
        ).reshape(common_shape)
        B_ort_value = onnxrt.OrtValue.ortvalue_from_numpy(B_data)  # noqa: N806

        Y_result = np.array(  # noqa: N806
            [
                546,
                561,
                576,
                552,
                564,
                576,
                39,
                42,
                45,
                1410,
                1461,
                1512,
                1362,
                1392,
                1422,
                201,
                222,
                243,
                2274,
                2361,
                2448,
                2172,
                2220,
                2268,
                363,
                402,
                441,
                2784,
                2850,
                2916,
                4362,
                4485,
                4608,
                1551,
                1608,
                1665,
                3540,
                3624,
                3708,
                5604,
                5763,
                5922,
                2037,
                2112,
                2187,
                4296,
                4398,
                4500,
                6846,
                7041,
                7236,
                2523,
                2616,
                2709,
                678,
                789,
                900,
                2892,
                3012,
                3132,
                4263,
                4494,
                4725,
                786,
                915,
                1044,
                3324,
                3462,
                3600,
                4911,
                5178,
                5445,
                894,
                1041,
                1188,
                3756,
                3912,
                4068,
                5559,
                5862,
                6165,
            ],
            np.float32,
        ).reshape(common_shape)

        sess = onnxrt.InferenceSession(
            get_name("sparse_to_dense_matmul.onnx"),
            providers=onnxrt.get_available_providers(),
        )
        res = sess.run_with_ort_values(["dense_Y"], {"sparse_A": A_ort_value, "dense_B": B_ort_value})
        self.assertEqual(len(res), 1)
        ort_value = res[0]
        self.assertTrue(isinstance(ort_value, onnxrt.OrtValue))
        self.assertTrue(ort_value.is_tensor())
        self.assertEqual(ort_value.data_type(), "tensor(float)")
        self.assertEqual(ort_value.shape(), common_shape)
        result = ort_value.numpy()
        self.assertEqual(list(result.shape), common_shape)
        self.assertTrue(np.array_equal(Y_result, result))


if __name__ == "__main__":
    unittest.main(verbosity=1)



import numpy as np
import onnx
from onnx import OperatorSetIdProto, TensorProto, helper, numpy_helper

hidden_size = 4
attention_head = 2
hidden_per_attention = 2

# Self-attention.
# Handle self-attention.
# MatMul->Add->Split->Reshape->Transpose->MatMul->Div->Mul->Sub->Softmax->Dropout->MatMul->Transpose->Reshape->MatMul->Add
#                  |->Reshape->Transpose->|                                        |
#                  |->Reshape->Transpose------------------------------------------>|

X = helper.make_tensor_value_info("input", TensorProto.FLOAT, ["batch", "seqlen", hidden_size])
X_mask = helper.make_tensor_value_info("mask", TensorProto.FLOAT, ["batch", 1, "seqlen", "seqlen"])
Y = helper.make_tensor_value_info("output", TensorProto.FLOAT, ["batch", "seqlen", hidden_size])

qkv_weight_np_vals = (0.01 * np.arange(hidden_size * (hidden_size * 3), dtype=np.float32)).reshape(
    (hidden_size, hidden_size * 3)
)
qkv_weight_initializer = numpy_helper.from_array(
    qkv_weight_np_vals, "transformer.attention.query_key_value.weight_transposed"
)

qkv_bias_np_vals = 0.01 * np.arange(hidden_size * 3, dtype=np.float32)
qkv_bias_initializer = numpy_helper.from_array(qkv_bias_np_vals, "transformer.attention.query_key_value.bias")

dense_weight_np_vals = (0.01 * np.arange(hidden_size * hidden_size, dtype=np.float32)).reshape(
    (hidden_size, hidden_size)
)
dense_weight_initializer = numpy_helper.from_array(
    dense_weight_np_vals, "transformer.attention.dense.weight_transposed"
)

dense_bias_np_vals = 0.01 * np.arange(hidden_size, dtype=np.float32)
dense_bias_initializer = numpy_helper.from_array(dense_bias_np_vals, "transformer.attention.dense.bias")

shape_val = np.array([0, 0, attention_head, hidden_per_attention], dtype=np.int64)
shape_initializer = numpy_helper.from_array(shape_val, "concat_shape_0")
shape_val1 = np.array([0, 0, attention_head, hidden_per_attention], dtype=np.int64)
shape_initializer1 = numpy_helper.from_array(shape_val1, "concat_shape_1")
shape_val2 = np.array([0, 0, attention_head, hidden_per_attention], dtype=np.int64)
shape_initializer2 = numpy_helper.from_array(shape_val2, "concat_shape_2")
shape_val3 = np.array([0, 0, hidden_size], dtype=np.int64)
shape_initializer3 = numpy_helper.from_array(shape_val3, "concat_shape_3")

matmul1 = helper.make_node("MatMul", ["input", qkv_weight_initializer.name], ["matmul1"], name="matmul1")
add1 = helper.make_node("Add", ["matmul1", qkv_bias_initializer.name], ["add1"], name="add1")
split = helper.make_node(
    "Split",
    ["add1"],
    ["mixed_query_layer", "mixed_key_layer", "mixed_value_layer"],
    name="split",
    axis=2,
)
reshape = helper.make_node(
    "Reshape",
    ["mixed_query_layer", shape_initializer.name],
    ["reshape"],
    name="reshape",
)
reshape1 = helper.make_node(
    "Reshape",
    ["mixed_key_layer", shape_initializer1.name],
    ["reshape1"],
    name="reshape1",
)
reshape2 = helper.make_node(
    "Reshape",
    ["mixed_value_layer", shape_initializer2.name],
    ["reshape2"],
    name="reshape2",
)
transpose = helper.make_node("Transpose", ["reshape"], ["transpose"], name="transpose", perm=[0, 2, 1, 3])
transpose1 = helper.make_node("Transpose", ["reshape1"], ["transpose1"], name="transpose1", perm=[0, 2, 3, 1])
transpose2 = helper.make_node("Transpose", ["reshape2"], ["transpose2"], name="transpose2", perm=[0, 2, 1, 3])
matmul2 = helper.make_node("MatMul", ["transpose", "transpose1"], ["matmul2"], name="matmul2")
# Use the mask input for below 3 nodes. This is different from the original GPT-2 model, but it's OK for te sub-graph test.
div = helper.make_node("Div", ["matmul2", "mask"], ["div"], name="div")
mul = helper.make_node("Mul", ["div", "mask"], ["mul"], name="mul")
sub = helper.make_node("Sub", ["mul", "mask"], ["sub"], name="sub")
softmax = helper.make_node("Softmax", ["sub"], ["softmax"], name="softmax", axis=3)
dropout1 = helper.make_node("Dropout", ["softmax"], ["dropout1"], name="dropout1")
matmul3 = helper.make_node("MatMul", ["dropout1", "transpose2"], ["matmul3"], name="matmul3")
transpose3 = helper.make_node("Transpose", ["matmul3"], ["transpose3"], name="transpose3", perm=[0, 2, 1, 3])
reshape3 = helper.make_node("Reshape", ["transpose3", shape_initializer3.name], ["reshape3"], name="reshape3")
matmul4 = helper.make_node("MatMul", ["reshape3", dense_weight_initializer.name], ["matmul4"], name="matmul4")
add2 = helper.make_node("Add", ["matmul4", dense_bias_initializer.name], ["add2"], name="add2")
dropout2 = helper.make_node("Dropout", ["add2"], ["dropout2"], name="dropout2")
# Add dummy Identity so during inference dropout2 can be removed for testing.
identity = helper.make_node("Identity", ["dropout2"], ["output"], name="identity")

# Create the graph (GraphProto)
graph_def = helper.make_graph(
    [
        matmul1,
        add1,
        split,
        reshape,
        reshape1,
        reshape2,
        transpose,
        transpose1,
        transpose2,
        matmul2,
        div,
        mul,
        sub,
        softmax,
        dropout1,
        matmul3,
        transpose3,
        reshape3,
        matmul4,
        add2,
        dropout2,
        identity,
    ],
    "self-attention-megatron-test-model",
    [X, X_mask],
    [Y],
    [
        qkv_weight_initializer,
        qkv_bias_initializer,
        dense_weight_initializer,
        dense_bias_initializer,
        shape_initializer,
        shape_initializer1,
        shape_initializer2,
        shape_initializer3,
    ],
)

opsets = []
onnxdomain = OperatorSetIdProto()
onnxdomain.version = 12
onnxdomain.domain = ""  # The empty string ("") or absence of this field implies the operator set that is defined as part of the ONNX specification.
opsets.append(onnxdomain)

msdomain = OperatorSetIdProto()
msdomain.version = 1
msdomain.domain = "com.microsoft"

opsets.append(msdomain)
kwargs = {}
kwargs["opset_imports"] = opsets

# Create the model (ModelProto)
model_def = helper.make_model(graph_def, producer_name="onnx-example", **kwargs)
onnx.save(model_def, "self_attention_megatron_basic_test.onnx")



import onnx
from onnx import TensorProto, helper


def MakeSubGraph(suffix, has_bias):  # noqa: N802
    mul_bottom_output = "mul_output" + suffix if has_bias else "output" + suffix
    nodes = [
        helper.make_node(
            "MatMulInteger",
            ["a_quantized", "b_quantized" + suffix, "a_zp", "b_zp" + suffix],
            ["matmul_output_int32" + suffix],
            "MatMulInteger" + suffix,
        ),
        helper.make_node(
            "Mul",
            ["a_scale", "b_scale" + suffix],
            ["multiplier" + suffix],
            "mul_right" + suffix,
        ),
        helper.make_node(
            "Cast",
            ["matmul_output_int32" + suffix],
            ["matmul_output_float" + suffix],
            "cast" + suffix,
            to=1,
        ),
        helper.make_node(
            "Mul",
            ["matmul_output_float" + suffix, "multiplier" + suffix],
            [mul_bottom_output],
            "mul_bottom" + suffix,
        ),
    ]

    if has_bias:
        nodes.extend(
            [
                helper.make_node(
                    "Add",
                    [mul_bottom_output, "bias" + suffix],
                    ["output" + suffix],
                    "bias_add" + suffix,
                ),
            ]
        )

    return nodes


def MakeInitializer(suffix):  # noqa: N802
    return [
        helper.make_tensor("b_quantized" + suffix, TensorProto.UINT8, [2, 3], [2, 4, 5, 6, 7, 8]),
        helper.make_tensor("b_zp" + suffix, TensorProto.UINT8, [], [128]),
        helper.make_tensor("b_scale" + suffix, TensorProto.FLOAT, [], [1.8]),
    ]


def GenerateModel(model_name):  # noqa: N802
    nodes = [
        helper.make_node(
            "DynamicQuantizeLinear",
            ["input"],
            ["a_quantized", "a_scale", "a_zp"],
            "DynamicQuantizeLinear",
        ),
    ]
    nodes.extend(MakeSubGraph("_1", True))
    nodes.extend(MakeSubGraph("_2", True))
    nodes.extend(MakeSubGraph("_3", False))

    initializers = []
    initializers.extend(MakeInitializer("_1"))
    initializers.extend(MakeInitializer("_3"))

    initializers.extend(
        [
            helper.make_tensor("bias_1", TensorProto.FLOAT, [3], [2, 4, 5]),
            helper.make_tensor("bias_2", TensorProto.FLOAT, [3, 3], [1, 2, 3, 4, 5, 6, 7, 8, 9]),
        ]
    )

    graph = helper.make_graph(
        nodes,
        "MatMulIntegerToFloat_fusion",  # name
        [  # inputs
            helper.make_tensor_value_info("input", TensorProto.FLOAT, [3, 2]),
            # matrix b corresponding inputs for subgraph 2
            helper.make_tensor_value_info("b_quantized_2", TensorProto.UINT8, [2, 3]),
            helper.make_tensor_value_info("b_zp_2", TensorProto.UINT8, [1]),
            helper.make_tensor_value_info("b_scale_2", TensorProto.FLOAT, [1]),
        ],
        [  # outputs
            helper.make_tensor_value_info("output_1", TensorProto.FLOAT, [3, 3]),
            helper.make_tensor_value_info("output_2", TensorProto.FLOAT, [3, 3]),
            helper.make_tensor_value_info("output_3", TensorProto.FLOAT, [3, 3]),
        ],
        initializers,
    )

    model = helper.make_model(graph)
    onnx.save(model, model_name)


if __name__ == "__main__":
    GenerateModel("matmul_integer_to_float.onnx")



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation.  All rights reserved.
# Licensed under the MIT License.  See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import os
import time
from typing import Any

import torch
from transformers import BartConfig, BartForConditionalGeneration, file_utils
from utils import export_helper


class DecoderInitWrapper(torch.nn.Module):
    """Initial part of Decoder wrapper.

    We split BART into three parts: Encoder plus initial part of Decodoer, main part of Decoder, and Beam Search.
    This module is the initial part of Decodoer which is introduced in `EncoderDecoderInit`.

    Attributes:
        model: the BART model.
    """

    def __init__(self, model):
        super().__init__()
        self.model = model

    def forward(
        self,
        last_hidden_state: torch.Tensor,
        decoder_input_ids: torch.Tensor,
        attention_mask: torch.Tensor,
    ):
        encoder_outputs = file_utils.ModelOutput()
        encoder_outputs["last_hidden_state"] = last_hidden_state
        encoder_outputs["hidden_states"] = None
        encoder_outputs["attentions"] = None
        return self.model(
            None,
            encoder_outputs=encoder_outputs,
            decoder_input_ids=decoder_input_ids,
            attention_mask=attention_mask,
            past_key_values=None,
            use_cache=True,
            return_dict=True,
        )


class EncoderDecoderInit(torch.nn.Module):
    """Encoder plus initial part of Decoder wrapper.

    We split BART into three parts: Encoder plus initial part of Decodoer, main part of Decoder, and Beam Search.
    This module is Encoder plus initial part of Decodoer.

    Attributes:
        encoder: the BART encoder.
        decoderwrapper: `DecoderInitWrapper`.
    """

    def __init__(self, encoder: torch.nn.Module, decoderwrapper: DecoderInitWrapper):
        super().__init__()
        self.encoder = encoder
        self.decoder_init = decoderwrapper

    def forward(
        self, encoder_input_ids: torch.Tensor, encoder_attention_mask: torch.Tensor, decoder_input_ids: torch.Tensor
    ):
        encoder_out = self.encoder(encoder_input_ids, encoder_attention_mask)
        encoder_output = encoder_out["last_hidden_state"]
        decoder_out = self.decoder_init(encoder_output, decoder_input_ids, encoder_attention_mask)
        present_self, present_cross = export_helper.group_by_self_and_cross(decoder_out.past_key_values)
        present = present_self + present_cross

        return decoder_out.logits, encoder_output, present


def _create_encoder_export(args, config: BartConfig):
    """Wrapper function to pass in args and config.

    This wrapper function exposes ONNX model output location and onnx version to users.

    Args:
        args: User input.
        config: BART config.
    """

    def _prepare_encoder_decoder_kwargs_for_generation(
        self, input_ids: torch.Tensor, model_kwargs, model_input_name: str | None = None
    ) -> dict[str, Any]:
        # retrieve encoder hidden states
        # 1. get encoder
        encoder = self.get_encoder()
        # 2. prepare encoder args and encoder kwargs from model kwargs
        irrelevant_prefix = ["decoder_", "use_cache"]
        encoder_kwargs = {
            argument: value
            for argument, value in model_kwargs.items()
            if not any(argument.startswith(p) for p in irrelevant_prefix)
        }
        # 3. make sure that encoder returns `ModelOutput`
        model_input_name = model_input_name if model_input_name is not None else self.main_input_name
        encoder_kwargs["return_dict"] = True
        encoder_kwargs[model_input_name] = input_ids
        model_kwargs["encoder_outputs"] = encoder(**encoder_kwargs)

        encoder_input_ids = input_ids.type(torch.int32)
        encoder_attention_mask = encoder_kwargs["attention_mask"].type(torch.int32)
        decoder_input_ids = (
            torch.ones((input_ids.shape[0], 1), dtype=torch.int32, device=input_ids.device)
            * config.decoder_start_token_id
        )

        wrapped_decoder = DecoderInitWrapper(self).eval()
        encdecinit = EncoderDecoderInit(encoder, wrapped_decoder)
        # use results from encdecinit here to compare with ORT.
        _, _, present = encdecinit(encoder_input_ids, encoder_attention_mask, decoder_input_ids)
        output_past_names = export_helper.get_input_names(present, encoder=True)

        # random name to use in dynamic axes
        sequence_length = "1"
        num_heads = str(config.encoder_attention_heads)
        hidden_size = str(config.d_model)
        head_size = str(config.encoder_attention_heads)

        dynamic_axes = {
            "encoder_input_ids": {0: "batch", 1: "sequence"},
            "encoder_attention_mask": {0: "batch", 1: "sequence"},
            "encoder_hidden_states": {0: "batch", 1: "sequence", 2: hidden_size},
            "decoder_input_ids": {0: "batch", 1: sequence_length},
            "logits": {0: "batch", 1: sequence_length},
        }

        for name in output_past_names:
            if "cross" in name:
                dynamic_axes[name] = {
                    0: "batch",
                    1: num_heads,
                    2: "sequence",
                    3: head_size,
                }
            else:  # self attention past state
                dynamic_axes[name] = {
                    0: "batch",
                    1: num_heads,
                    2: sequence_length,
                    3: head_size,
                }

        model_path = os.path.join(args.output, "edinit.onnx")
        torch.onnx.export(
            encdecinit,
            (encoder_input_ids, encoder_attention_mask, decoder_input_ids),
            model_path,
            opset_version=args.opset_version,
            do_constant_folding=False,
            input_names=["encoder_input_ids", "encoder_attention_mask", "decoder_input_ids"],
            output_names=["logits", "encoder_hidden_states", *output_past_names],
            dynamic_axes=dynamic_axes,
            export_params=True,
            verbose=True,
        )

        return model_kwargs

    return _prepare_encoder_decoder_kwargs_for_generation


def export_encoder(args):
    """Export BART encoder to ONNX.

    By replacing the inner function of `_prepare_encoder_decoder_kwargs_for_generation`,
    we export encoder model into ONNX with the usage of model.genration()

    Args:
        args: User input.

    Return:
        Encoder ONNX model in the given output directory, or under onnx_models folder.
    """
    beam = args.num_beams
    min_length = args.min_length
    max_length = args.max_length
    repetition_penalty = args.repetition_penalty
    no_repeat_ngram_size = args.no_repeat_ngram_size

    config, tokenizer = export_helper.initialize_config(args)

    with torch.no_grad():
        model, input_data = export_helper.initialize_model(config, tokenizer, args)
        start_time = time.time()
        model._prepare_encoder_decoder_kwargs_for_generation = _create_encoder_export(args, config).__get__(
            model, BartForConditionalGeneration
        )

        pred_ids = model.generate(
            input_data,
            decoder_start_token_id=tokenizer.eos_token_id,
            num_beams=beam,
            num_return_sequences=beam,
            min_length=min_length,
            max_length=max_length,
            repetition_penalty=repetition_penalty,
            no_repeat_ngram_size=no_repeat_ngram_size,
        )
        time_cost = time.time() - start_time
        print(f"--- {time_cost} seconds ---")
        print(tokenizer.decode(pred_ids[0], skip_special_tokens=True, clean_up_tokenization_spaces=False))



import onnx
from onnx import OperatorSetIdProto, TensorProto, helper

X = helper.make_tensor_value_info("input", TensorProto.FLOAT, ["batch", "seqlen", 128])
unsqueezed_masked_lm_positions = helper.make_tensor_value_info(
    "unsqueezed_masked_lm_positions",
    TensorProto.INT64,
    ["batch", "dynamic_prediction_count", 1],
)
Y = helper.make_tensor_value_info("output", TensorProto.FLOAT, ["batch", "dynamic_prediction_count", 128])
nodes = []

# case 1
gelu1 = helper.make_node("Gelu", ["input"], ["gelu_1"], name="gelu_1", domain="com.microsoft")
nodes.append(gelu1)

gathernd1 = helper.make_node(
    "GatherND",
    ["gelu_1", "unsqueezed_masked_lm_positions"],
    ["gathernd_out"],
    name="gathernd_1",
    batch_dims=1,
)
nodes.append(gathernd1)

identity = helper.make_node("Identity", ["gathernd_out"], ["output"], name="identity")
nodes.append(identity)

graph_def = helper.make_graph(nodes, "test-model", [X, unsqueezed_masked_lm_positions], [Y])

opsets = []
onnxdomain = OperatorSetIdProto()
onnxdomain.version = 12
onnxdomain.domain = ""  # The empty string ("") or absence of this field implies the operator set that is defined as part of the ONNX specification.
opsets.append(onnxdomain)

msdomain = OperatorSetIdProto()
msdomain.version = 1
msdomain.domain = "com.microsoft"

opsets.append(msdomain)
kwargs = {}
kwargs["opset_imports"] = opsets

model_def = helper.make_model(graph_def, producer_name="onnx-example", **kwargs)

onnx.save(model_def, "gathernd_gelu.onnx")



# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.

# This code is used to generate the test cases for the AffineGrid operator
# in onnxruntime/test/providers/cpu/tensor/affine_grid_test.cc

import argparse

import numpy as np
import torch
from torch.nn.functional import affine_grid

opset_version = 20
parser = argparse.ArgumentParser(description="Generate test cases for the AffineGrid operator.")
parser.add_argument("--dim", type=int, choices=[2, 3], help="Dimension of the test cases (2 or 3)")
args = parser.parse_args()

if args.dim is None or args.dim == 2:
    align_corners_options = [False, True]
    angles = [10, 60]
    translations = [np.array([0.3, -0.5]), np.array([-0.5, -0.5])]
    scales = [np.array([1.5, 0.5]), np.array([3.0, 5.5])]
    sizes = [[1, 1, 3, 2], [2, 10, 2, 3]]
    test_count = 0

    for align_corners in align_corners_options:
        for angle, translation, scale in zip(angles, translations, scales, strict=False):
            for size in sizes:
                theta = np.array([], dtype=np.float32)
                for _ in range(size[0]):
                    angle_radian = (angle / 180.0) * np.pi
                    theta = np.append(
                        theta,
                        [
                            np.cos(angle_radian) * scale[0],
                            -np.sin(angle_radian),
                            translation[0],
                            np.sin(angle_radian),
                            np.cos(angle_radian) * scale[1],
                            translation[1],
                        ],
                    )
                theta = theta.reshape(size[0], 2, 3)
                theta = torch.Tensor(theta)
                grid = affine_grid(theta, size, align_corners=align_corners)

                # Print the C++ code for the test case
                print(f"TEST(AffineGridTest, test_2d_{test_count}) {{")
                print(f'  OpTester test("AffineGrid", {opset_version});')
                print(f'  test.AddAttribute("align_corners", (int64_t){1 if align_corners else 0});')
                print(
                    f'  test.AddInput<float>("theta", {{{theta.shape[0]}, {theta.shape[1]}, {theta.shape[2]}}}, {{{", ".join([f"{x:.6f}f" for x in theta.flatten()])}}});'
                )
                print(
                    f'  test.AddInput<int64_t>("size", {{{len(size)}}}, {{{size[0]}, {size[1]}, {size[2]}, {size[3]}}});'
                )
                print(
                    f'  test.AddOutput<float>("grid", {{{size[0]}, {size[2]}, {size[3]}, 2}}, {{{", ".join([f"{x:.4f}f" for x in grid.flatten()])}}});'
                )
                print("  test.Run();")
                print("}\n")
                test_count += 1


if args.dim is None or args.dim == 3:
    align_corners_options = [False, True]
    angles = [[10, 20], [60, -30]]
    translations = [np.array([0.3, -0.5, 1.8]), np.array([-0.5, -0.5, 0.3])]
    scales = [np.array([1.5, 2.0, 0.5]), np.array([0.3, 3.0, 5.5])]
    sizes = [[1, 1, 3, 2, 2], [2, 10, 2, 2, 3]]
    test_count = 0

    for align_corners in align_corners_options:
        for angle, translation, scale in zip(angles, translations, scales, strict=False):
            for size in sizes:
                theta = np.array([], dtype=np.float32)
                for _ in range(size[0]):
                    angle_radian_x = (angle[0] / 180.0) * np.pi
                    angle_radian_y = (angle[1] / 180.0) * np.pi
                    rot_matrix_x = np.array(
                        [
                            [1, 0, 0],
                            [0, np.cos(angle_radian_x), -np.sin(angle_radian_x)],
                            [0, np.sin(angle_radian_x), np.cos(angle_radian_x)],
                        ]
                    )
                    rot_matrix_y = np.array(
                        [
                            [np.cos(angle_radian_y), 0, np.sin(angle_radian_y)],
                            [0, 1, 0],
                            [-np.sin(angle_radian_y), 0, np.cos(angle_radian_y)],
                        ]
                    )
                    rot_matrix = np.matmul(rot_matrix_x, rot_matrix_y)
                    rot_matrix = rot_matrix * scale.reshape(3, 1)
                    rot_matrix = np.append(rot_matrix, np.reshape(translation, (3, 1)), axis=1)
                    theta = np.append(theta, rot_matrix.flatten())
                theta = theta.reshape(size[0], 3, 4)
                theta = torch.Tensor(theta)
                grid = affine_grid(theta, size, align_corners=align_corners)

                # Print the C++ code for the test case
                print(f"TEST(AffineGridTest, test_3d_{test_count}) {{")
                print(f'  OpTester test("AffineGrid", {opset_version});')
                print(f'  test.AddAttribute("align_corners", (int64_t){1 if align_corners else 0});')
                print(
                    f'  test.AddInput<float>("theta", {{{theta.shape[0]}, {theta.shape[1]}, {theta.shape[2]}}}, {{{", ".join([f"{x:.6f}f" for x in theta.flatten()])}}});'
                )
                print(
                    f'  test.AddInput<int64_t>("size", {{{len(size)}}}, {{{size[0]}, {size[1]}, {size[2]}, {size[3]}, {size[4]}}});'
                )
                print(
                    f'  test.AddOutput<float>("grid", {{{size[0]}, {size[2]}, {size[3]}, {size[4]}, 3}}, {{{", ".join([f"{x:.4f}f" for x in grid.flatten()])}}});'
                )
                print("  test.Run();")
                print("}\n")
                test_count += 1



#!/usr/bin/python3
import argparse
import os


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("--src_root", required=True, help="input symbol file")
    parser.add_argument("--output", required=True, help="output file")
    parser.add_argument("--output_source", required=True, help="output file")
    parser.add_argument("--version_file", required=True, help="VERSION_NUMBER file")
    parser.add_argument("--style", required=True, choices=["gcc", "vc", "xcode"])
    parser.add_argument("--config", required=True, nargs="+")
    return parser.parse_args()


args = parse_arguments()
print(f"Generating symbol file for {args.config!s}")
with open(args.version_file) as f:
    VERSION_STRING = f.read().strip()

print(f"VERSION:{VERSION_STRING}")

symbols = set()
for c in args.config:
    file_name = os.path.join(args.src_root, "core", "providers", c, "symbols.txt")
    with open(file_name) as file:
        for line in file:
            line = line.strip()  # noqa: PLW2901
            if line in symbols:
                print("dup symbol: %s", line)
                exit(-1)
            symbols.add(line)
symbols = sorted(symbols)

symbol_index = 1
with open(args.output, "w") as file:
    if args.style == "vc":
        file.write("LIBRARY\n")
        file.write("EXPORTS\n")
    elif args.style == "xcode":
        pass  # xcode compile don't has any header.
    else:
        file.write(f"VERS_{VERSION_STRING} {{\n")
        file.write(" global:\n")

    for symbol in symbols:
        if args.style == "vc":
            file.write(f" {symbol} @{symbol_index}\n")
        elif args.style == "xcode":
            file.write(f"_{symbol}\n")
        else:
            file.write(f"  {symbol};\n")
        symbol_index += 1

    if args.style == "gcc":
        file.write(" local:\n")
        file.write("    *;\n")
        file.write("};   \n")

with open(args.output_source, "w") as file:
    file.write("#include <onnxruntime_c_api.h>\n")
    for c in args.config:
        # WinML adapter should not be exported in platforms other than Windows.
        # Exporting OrtGetWinMLAdapter is exported without issues using .def file when compiling for Windows
        # so it isn't necessary to include it in generated_source.c

        # external symbols are removed, xnnpack ep will be created via the standard ORT API.
        # https://github.com/microsoft/onnxruntime/pull/11798
        if c not in (
            "vitisai",
            "winml",
            "cuda",
            "rocm",
            "migraphx",
            "qnn",
            "snpe",
            "xnnpack",
            "cann",
            "dnnl",
            "tensorrt",
            "azure",
            "webgpu",
        ):
            file.write(f"#include <core/providers/{c}/{c}_provider_factory.h>\n")
    file.write("void* GetFunctionEntryByName(const char* name){\n")
    for symbol in symbols:
        if symbol != "OrtGetWinMLAdapter":
            file.write(f'if(strcmp(name,"{symbol}") ==0) return (void*)&{symbol};\n')
    file.write("return NULL;\n")
    file.write("}\n")
